package bioutil;

import sim.app.socialagents.CommSocialAgent;


/**
 * Class for representing the state
 * of the agent searching for a target.
 * This agent will follow the morphogen
 * of others to reach the target.
 * 
 * @author Adelein Rodriguez
 */
public class SearchState{
	
	public CommSocialAgent agent;
	
	private boolean hasReachedGoal=false;
	//	This is what this agent knows about gradients in its
	//	area from its neighbors.
	private Morphogen searchGuideGradient; 
	// We need to keep track of what neighbor we queried
	// before so that we dont get stuck following the same one.
	private CommSocialAgent previousNeighbor;
	// Also, how many times are we allowed to use
	// this guys as guide.
	public int maxFanaticTime = 5 ;
	// How many times have we followed the same neigbhor 
	// consecutively?
	private int timeOnGuidedByPreviousNeighbor = 0; 

	/*	
	 * Initially when an agent is designated
	 * as the search agent, its only guide gradient is its
	 * own, then it will change as it queries to its 
	 * neighbors.
	*/
	public SearchState(CommSocialAgent agent, Morphogen activeGradient){
	
		this.agent=agent;
		this.searchGuideGradient=activeGradient;
	}
	/* It is ok to follow to use the information of a neighbor if
	 * it is better than our information, or more than we know (if we kno
	 * nothing) of it is the same but comes from a different neighbor
	 * than our previous.
	*/
	public boolean okToFollowNeighbhor(CommSocialAgent neighbor){
		
		//	This agent has information about the gradient
		//	but its neighbor has better information.
		boolean neighborKnowsBetter = neighbor.activeGradient != null && this.searchGuideGradient!=null &&
		neighbor.activeGradient.value > this.searchGuideGradient.value;
		//	This agent has no information about the gradient at all
		//	thus it is better to have some than have no information.
		boolean neighborKnowsMore = neighbor.activeGradient!=null && this.searchGuideGradient==null &&
		neighbor.activeGradient.value > this.searchGuideGradient.value;
		
		boolean neighborKnowsSame = neighbor.activeGradient!=null && this.searchGuideGradient==null &&
		neighbor.activeGradient.value == this.searchGuideGradient.value;
		
		boolean noPrevNeighbor = (this.previousNeighbor==null);
		boolean prevWasDifferent = noPrevNeighbor || (this.previousNeighbor.id != neighbor.id);
		

		//	If we have used this neighbor before consecutively,
		//	Have we overused it?
		boolean sameButNotOverusedNeighbor = (!prevWasDifferent && 
										  this.timeOnGuidedByPreviousNeighbor < this.maxFanaticTime);

		if(prevWasDifferent){
			this.timeOnGuidedByPreviousNeighbor=0;
			//this.previousNeighbor=neighbor;
			//this.searchGuideGradient=neighbor.activeGradient;
		}
		if(neighborKnowsBetter || neighborKnowsMore){
			this.previousNeighbor=neighbor;
			this.searchGuideGradient=neighbor.activeGradient;
			return true;
		}else if(sameButNotOverusedNeighbor){
			this.previousNeighbor=neighbor;
			this.searchGuideGradient=neighbor.activeGradient;
			this.timeOnGuidedByPreviousNeighbor++;
			return true;
		}else if(neighborKnowsSame && prevWasDifferent){
			this.previousNeighbor=neighbor;
			this.searchGuideGradient=neighbor.activeGradient;
			return true;
		}
	
		return false;
		
	}
	
	public boolean hasReachedGoal(){
		return hasReachedGoal;
	}	
	// Setter and getters;
	public void setSearchGuideGradient(Morphogen newGuide){
		this.searchGuideGradient=newGuide;
	}
	public Morphogen getSearchGuideGradient(){
		return this.searchGuideGradient;
	}

}