package sim.app.socialagents;

import sim.util.*;
import java.text.DecimalFormat;

public abstract class AgentController{

	public static int NUM_CONDITIONS = 0;
	public static int NUM_NG_PARAMS = 2;
	public static int NUM_G_PARAMS = 6;
	public static int NUM_PARAMS = AgentController.NUM_NG_PARAMS + AgentController.NUM_G_PARAMS;

	//public static int BOOL_ON_AOI = 0;

	public static int NG_RANDOMNESS = 0;
	public static int NG_COMFORT    = 1;

	public static int G_AVOID_AGENT         = 2;
	public static int G_AVOID_OBSTACLE      = 3;
	public static int G_MOMENTUM            = 4;
	public static int G_GOTO_AOI            = 5;
	public static int G_FOLLOW_OBSTACLE     = 6;
	public static int G_GOTO_MESSAGE_POINT  = 7;

	protected double[][] nonGoalParams;
	protected Role[] roles;

	public void addControl(boolean[] conditions, double[] parameters){

		int conditionIndex = this.getConditionIndex(conditions);

		for(int i = 0; i < AgentController.NUM_NG_PARAMS; i++){
			this.nonGoalParams[conditionIndex][i] = parameters[i];
		}

		this.roles[conditionIndex] = new Role(AgentController.NUM_G_PARAMS);

		this.roles[conditionIndex].addGoal(new AvoidAgentGoal(), parameters[AgentController.G_AVOID_AGENT]);
		this.roles[conditionIndex].addGoal(new AvoidObstacleGoal(), parameters[AgentController.G_AVOID_OBSTACLE]);
		this.roles[conditionIndex].addGoal(new MomentumGoal(), parameters[AgentController.G_MOMENTUM]);
		this.roles[conditionIndex].addGoal(new GotoAOIGoal(), parameters[AgentController.G_GOTO_AOI]);
		this.roles[conditionIndex].addGoal(new FollowObstacleGoal(), parameters[AgentController.G_FOLLOW_OBSTACLE]);
		this.roles[conditionIndex].addGoal(new CommGotoMessagePointGoal(), parameters[AgentController.G_GOTO_MESSAGE_POINT]);

	}
	public void changeParameters(double[] parameters){
		int conditionIndex = 0;
		for(int i = 0; i < AgentController.NUM_NG_PARAMS; i++){
			this.nonGoalParams[conditionIndex][i] = parameters[i];
		}
		
		this.roles[conditionIndex] = new Role(AgentController.NUM_G_PARAMS);

		this.roles[conditionIndex].addGoal(new AvoidAgentGoal(), parameters[AgentController.G_AVOID_AGENT]);
		this.roles[conditionIndex].addGoal(new AvoidObstacleGoal(), parameters[AgentController.G_AVOID_OBSTACLE]);
		this.roles[conditionIndex].addGoal(new MomentumGoal(), parameters[AgentController.G_MOMENTUM]);
		this.roles[conditionIndex].addGoal(new GotoAOIGoal(), parameters[AgentController.G_GOTO_AOI]);
		this.roles[conditionIndex].addGoal(new FollowObstacleGoal(), parameters[AgentController.G_FOLLOW_OBSTACLE]);
		this.roles[conditionIndex].addGoal(new CommGotoMessagePointGoal(), parameters[AgentController.G_GOTO_MESSAGE_POINT]);
		
		
	}

	public void addControl(SingleControl control){

		this.addControl(control.getConditions(), control.getParameters());

	}

	public int getConditionIndex(boolean[] conditions){

		int index = 0;

		//index += (int)Math.pow(2, AgentController.BOOL_ON_AOI) * (conditions[AgentController.BOOL_ON_AOI]?1:0);

		return index;

	}

	public double getNonGoalParam(Agent agent, int index){

		boolean[] currentCondition = new boolean[AgentController.NUM_CONDITIONS];
		int curConditionIndex;

		// for each condition variable in AgentController, set the appropriate currentCondition
		//currentCondition[AgentController.BOOL_ON_AOI] = agent.getOnAOI();

		curConditionIndex = this.getConditionIndex(currentCondition);

		return this.nonGoalParams[curConditionIndex][index];
	}

	/* returns a header for printing out an agent controller object
	 **/
	public static String getHeader(){

		return "#RANDOMNESS COMFORT AVOID_AGENT AVOID_OBS MOMENTUM GOTO_AOI FOLLOW_OBS GOTO_MESSAGE_POINT";
                                                                                               
	}

	public static String getString(double[] params){

		DecimalFormat df = new DecimalFormat("0.00000");
		String ret;

		ret =  df.format(params[0]) + "     ";
		ret += df.format(params[1]) + " ";
		ret += df.format(params[2]) + "     ";
		ret += df.format(params[3]) + "   ";
		ret += df.format(params[4]) + "  ";
		ret += df.format(params[5]) + "  ";
		ret += df.format(params[6]) + "    ";
		ret += df.format(params[7]);

		return ret;

	}

	public abstract MutableDouble2D getNewVelocity(CommAgent agent);

}
