/* This class computes a simple vector in the opposite direction of a nearby obstacle
 * This will be used to avoid agent-obstacle collisions
 **/
package sim.app.socialagents;

import sim.util.*;

public class AvoidObstacleGoal extends Goal{

	public AvoidObstacleGoal(){

	}

	/* simply creates a vector pointing away from the obstaclePosition vector
	 **/
	public MutableDouble2D getVectorToGoal(CommAgent agent){

		MutableDouble2D agentPosition = new MutableDouble2D(agent.positionx, agent.positiony);
		MutableDouble2D vector, closestPoint;
		Double2D obstaclePosition;
		double minDist;
		double comfortZoneSq = (agent.radius*agent.radius) * (agent.comfortZone * agent.comfortZone);

		closestPoint = new MutableDouble2D(Double.MAX_VALUE, Double.MAX_VALUE);
		minDist = Double.MAX_VALUE;

		// find the closest obstacle point
		for(int i = 0; i < agent.obstaclePositions.numObjs; i++){

			obstaclePosition = (Double2D)agent.obstaclePositions.objs[i];

			if(agentPosition.distanceSq(obstaclePosition) < minDist){
				closestPoint = new MutableDouble2D(obstaclePosition);
				minDist = agentPosition.distanceSq(obstaclePosition);
			}

		}
		//System.out.println("Num of obstacles: "+agent.nearbyObstacles.size());
		/* if the closest obstacle is getting within the agent's comfort zone, then ignore it
		 * otherwise, create a vector pointing away from that obstacle
		 **/
		if(minDist > comfortZoneSq){
			return new MutableDouble2D(0.0, 0.0);
		}else{
			//System.out.println("Detected Obstacle");
			//System.out.println("Old velocity: " +agent.velocityx+","+agent.velocityy);
			vector = new MutableDouble2D();

			vector.subtract(agentPosition, closestPoint);

			if(vector.lengthSq() != 0.0){
				vector.normalize();
			}
			//System.out.println("New velocity: " +vector.x+","+vector.y);
			return vector;
		}

	}

}
