package sim.app.socialagents;

import sim.engine.*;
import sim.portrayal.*;
import sim.util.*;
import java.awt.*;
import java.awt.geom.*;
import java.util.*;

public abstract class CommAgent extends Agent{


	public CommAgent(Double2D vel, Double2D pos, double diam, double sense, Color col, double speed){
		
		super(vel,pos,diam,sense,col,speed);

	}

	/* This simply draws an oval for the Agent
	 * 
	 * Note that no color is set here.  The subclasses should set the agents' color
	 **/
	public void draw(Object object, Graphics2D graphics, DrawInfo2D info){

		// values for the circle
		final double width = info.draw.width * this.diameter;
		final double height = info.draw.height * this.diameter;
		final int x = (int)(info.draw.x - width / 2.0);
		final int y = (int)(info.draw.y - height / 2.0);
		final int w = (int)(width);
		final int h = (int)(height);

		if(this.alive){

			// values for the direction
			final double magn = Math.sqrt(this.velocityx * this.velocityx + this.velocityy * this.velocityy);
			final int xline, yline;

			if(magn != 0.0){
				xline = (int)(info.draw.x + (velocityx/magn) * (info.draw.width * this.radius));
				yline = (int)(info.draw.y + (velocityy/magn) * (info.draw.height * this.radius));
			}else{
				xline = (int)(info.draw.x);
				yline = (int)(info.draw.y);
			}

			// values for the sensor
			final double sensorWidth = info.draw.width * this.sensorDist*2.0;
			final double sensorHeight = info.draw.height * this.sensorDist*2.0;
			final int sensorX = (int)(info.draw.x - sensorWidth/2.0);
			final int sensorY = (int)(info.draw.y - sensorHeight/2.0);
			final int sensorW = (int)(sensorWidth);
			final int sensorH = (int)(sensorHeight);

			// draw the sensor circle
			graphics.setColor(new Color(63, 63, 63));
			graphics.drawOval(sensorX, sensorY, sensorW, sensorH);

			// draw the AV centered on the origin
			if(this.highlighted){
				graphics.setColor(this.highlightColor);
			}else{
				graphics.setColor(this.color);
			}
			graphics.fillOval(x, y, w, h);

			// draw its path
			if(this.drawPath && this.pathIndex >= 2){

				int x1, y1, x2, y2;

				if(this.pathIndex < this.maxPathLength){
					for(int i = 0; i < this.pathIndex-1; i++){
						graphics.setColor(new Color(255 - (int)(255.0/this.pathIndex * i), 255 - (int)(255.0/this.pathIndex * i), 255 - (int)(255.0/this.pathIndex * i)));
						x1 = (int)(info.draw.width * this.path[i].x);
						y1 = (int)(info.draw.height * this.path[i].y);
						x2 = (int)(info.draw.width * this.path[i+1].x);
						y2 = (int)(info.draw.height * this.path[i+1].y);
						graphics.drawLine(x1, y1, x2, y2);
					}
				}else{
					/*for(int i = this.pathIndex, j = 0; i < (this.pathIndex + this.maxPathLength) - 1; i++, j++){
						graphics.setColor(new Color(255 - (int)(255.0/this.maxPathLength * j), 255 - (int)(255.0/this.pathIndex * i), 255 - (int)(255.0/this.pathIndex * i)));
						x1 = (int)(info.draw.width * this.path[i % this.maxPathLength].x);
						y1 = (int)(info.draw.height * this.path[i % this.maxPathLength].y);
						x2 = (int)(info.draw.width * this.path[(i+1) % this.maxPathLength].x);
						y2 = (int)(info.draw.height * this.path[(i+1) % this.maxPathLength].y);
						graphics.drawLine(x1, y1, x2, y2);
					}*/
				}

			}

			// draw lines to nearby agents
			graphics.setColor(Color.red);
			if(this.agentPositions != null){
				Double2D otherAgentPosition;
				for(int i = 0; i < this.agentPositions.numObjs; i++){
					otherAgentPosition = (Double2D)this.agentPositions.objs[i];
					graphics.drawLine((int)info.draw.x, (int)info.draw.y, (int)(info.draw.width * otherAgentPosition.x), (int)(info.draw.height * otherAgentPosition.y));
				}
			}

			// draw lines to nearby obstacles
			graphics.setColor(Color.orange);
			if(this.obstaclePositions != null){
				Double2D obstaclePos;
				for(int i = 0; i < this.obstaclePositions.numObjs; i++){
					obstaclePos = (Double2D)(this.obstaclePositions.objs[i]);
					graphics.drawLine((int)info.draw.x, (int)info.draw.y, (int)(info.draw.width * obstaclePos.x), (int)(info.draw.height * obstaclePos.y));
				}
			}

			// draw its direction
			graphics.setColor(Color.white);
			graphics.drawLine((int)info.draw.x, (int)info.draw.y, xline, yline);

		}else{

			graphics.setColor(Color.black);
			graphics.fillOval(x, y, w, h);

		}


	}

	

	/* simply updates the agent's position based off of its velocity and current position
	 **/
	public void step(SimState state){

		CommSocialSim sim = (CommSocialSim)state;

		/* the following is for the path drawing
		 * only record the Agent's position for the path each 25 time steps
		 **/
		if(sim.schedule.getSteps() % 25 == 0){

			this.path[this.pathIndex % this.maxPathLength] = new Double2D(this.positionx, this.positiony);
			this.pathIndex++;
		}

		// get the positions of nearby agents, obstacles and AOIs
		this.getNearbyAgentPositions(state);
		this.getNearbyObstaclePositions(state);
		this.getNearbyAOIPositions(state);

		this.activated = true;

		/* reverse direction if a wall is encountered
		 **/
		if((this.positionx - this.radius <= 0.0 && this.velocityx < 0.0) || 
		   (this.positionx + this.radius >= sim.fieldWidth && this.velocityx > 0.0)){
			this.velocityx *= -1.0;
		}

		if((this.positiony - this.radius <= 0.0 && this.velocityy < 0.0) || 
		   (this.positiony + this.radius >= sim.fieldHeight && this.velocityy > 0.0)){
			this.velocityy *= -1.0;
		}

	}

	public void moveOneStep(SimState state){

		CommSocialSim sim = (CommSocialSim)state;

		/* update the position in the Continuous2D container
		 * also, update the position within the agent
		 **/
		Double2D newpos = new Double2D(this.positionx + this.velocityx, this.positiony + this.velocityy);
		sim.agents.setObjectLocation(this,newpos);

		this.updatePosition(newpos);

	}

	public void updatePosition(Double2D pos){
		this.positionx = pos.x;
		this.positiony = pos.y;
	}

	public void updateVelocity(double x, double y){
		this.velocityx = x;
		this.velocityy = y;
	}

	/* Update the agent's list of neighboring obstacle positions
	 **/
	Bag nearbyObstacles = new Bag(); 
	protected void getNearbyObstaclePositions(SimState state){

		CommSocialSim sim = (CommSocialSim)state;

		// obtain a bag containing possible obstacles within sensor distance
		//final Bag nearbyObstacles = sim.obstacles.getObjectsWithinDistance(new Double2D(this.positionx,this.positiony),this.getSensorDist(),false,false);
		sim.obstacles.getObjectsWithinDistance(new Double2D(this.positionx,this.positiony),this.getSensorDist(),false,false,nearbyObstacles);
		this.obstaclePositions = new Bag();
		//this.obstaclePositions.clear(); 
		Double2D closestPoint;
		//System.out.println("Num of obstacles: "+nearbyObstacles.numObjs);
		for(int i = 0; i < nearbyObstacles.numObjs; i++){

			final Obstacle obstacle = (Obstacle)(nearbyObstacles.objs[i]);

			// skip the region if it is not active
			if(!obstacle.isActive()) continue;

			closestPoint = obstacle.getClosestPoint(this.positionx, this.positiony, this.sensorDist);

			/* checks to see if the point is inside the region, if it is
			 *   then the agent dies
			 **/
			
			double dist = Region.distance(positionx,positiony,closestPoint.x, closestPoint.y);
			
			/*if(obstacle.pointInsideRegion(this.positionx, this.positiony) || this.radius >= dist) {
				this.setAlive(false);
			}*/

			if(closestPoint.x < Double.MAX_VALUE){
				this.obstaclePositions.add(closestPoint);
			}

		}

	}

	/* Update the agent's list of neighboring AOI positions
	 **/
	Bag nearbyAOIs = new Bag(); 
	protected void getNearbyAOIPositions(SimState state){

		CommSocialSim sim = (CommSocialSim)state;

		// obtain a bag containing possible obstacles within sensor distance
		//final Bag nearbyAOIs = sim.areasOfInterest.getObjectsWithinDistance(new Double2D(this.positionx,this.positiony),this.getSensorDist(),false,false);
		sim.areasOfInterest.getObjectsWithinDistance(new Double2D(this.positionx,this.positiony),this.getSensorDist(),false,false,nearbyAOIs);
		Double2D closestPoint;
		this.aoiPositions = new Bag();
		//this.aoiPositions.clear(); 
		
		this.onAOI = false;
		this.aoiCurrentlyOn = -1;

		for(int i = 0; i < nearbyAOIs.numObjs; i++){

			final AreaOfInterest aoi = (AreaOfInterest)(nearbyAOIs.objs[i]);

			// skip the region if it is not active
			if(!aoi.isActive()) continue;


			if(aoi.pointInsideRegion(this.positionx, this.positiony)){

				this.aoiPositions.add(new Double2D(this.positionx, this.positiony));

				this.onAOI = true;
				this.timeOverAOI++;
				this.aoisSeen.add(new Integer(aoi.getID()));
				this.aoiCurrentlyOn = aoi.getID();

			}else{
				closestPoint = aoi.getClosestPoint(this.positionx, this.positiony, this.sensorDist);

				if(closestPoint.x < Double.MAX_VALUE){
					this.aoiPositions.add(closestPoint);
				}

			}

		}

	}

	/* Update the agent's list of neighboring agent positions
	 **/
	Bag nearbyAgents = new Bag(); 
	protected void getNearbyAgentPositions(SimState state){
		
		CommSocialSim sim = (CommSocialSim)state;

		// obtain a bag containing possible neighbors within sensor distance
		//final Bag nearbyAgents = sim.agents.getObjectsWithinDistance(new Double2D(this.positionx,this.positiony),this.getSensorDist(),false,false);
		sim.agents.getObjectsWithinDistance(new Double2D(this.positionx,this.positiony),this.getSensorDist(),false,true,nearbyAgents);
		
		this.agentPositions = new Bag();
		
		
		// iterate through each nearby agent and check whether or not it is within sensor range of this
		for(int i = 0; i < nearbyAgents.numObjs; i++){

			final CommAgent otherAgent = (CommAgent)(nearbyAgents.objs[i]);
			final double dist = Math.sqrt((otherAgent.positionx-this.positionx)*(otherAgent.positionx-this.positionx)+(otherAgent.positiony-this.positiony)*(otherAgent.positiony-this.positiony));

			// Check to see if the agent is within the sensor range.
			// if two agents are on top of eachother, then kill them both
			 
			if(dist < this.radius + otherAgent.radius && dist > 0.0 && otherAgent.isAlive() && otherAgent.activated){
				//this.setAlive(false);
				//otherAgent.setAlive(false);
			}else if(dist < this.sensorDist + otherAgent.radius && otherAgent.isAlive() && dist != 0.0){
				this.agentPositions.add(new Double2D(otherAgent.positionx, otherAgent.positiony));
			}

		}

	}


}

