package sim.app.socialagents;

import sim.engine.*;
import sim.util.*;
import java.util.*;
import java.io.*;
import java.text.*;

public class CommDataCollector implements Steppable{

	private int collectInterval;

	/* the data that is collected each interval are:
	 * AVS_ALIVE   AVS_ON_AOIS    %AOIS_COVERED    AVG_AOIS_SEEN_BY_AGENTS    AVG_%_TIME_ON_AOIS
	 **/
	private double[][] dataAvg;
	private double[][] dataStdDev;

	/* runLength - the number of time steps to record the data for
	 * interval - the data is written to the output file every 'interval' time steps
	 *            this must be >= 1
	 **/
	public CommDataCollector(int runLength, int interval){

		this.collectInterval = interval;
		this.dataAvg = new double[runLength/interval + 1][6];
		this.dataStdDev = new double[runLength/interval + 1][6];

	}

	/* records all of the important data for the run
	 **/
	public void step(SimState state){

		int index = (int)state.schedule.getSteps() / this.collectInterval;

		// if the simulation is not on the current step, exit this function
		if(state.schedule.getSteps() % this.collectInterval != 0 || index >= this.dataAvg.length) return;

		CommSocialSim sim = (CommSocialSim)state;

		this.dataAvg[index][0] = state.schedule.getSteps();
		this.dataAvg[index][1] = sim.numAgentsAlive();
		this.dataAvg[index][2] = sim.numAgentsOnAOIs();
		this.dataAvg[index][3] = sim.percentageAOIsCovered(); // %AOIS_COVERED
		this.dataAvg[index][4] = sim.avgAOIsSeenByAgents();
		this.dataAvg[index][5] = sim.avgPercentTimeOverAOIs();
	}

	/* returns an array that contains the max value of each column in the data array
	 **/
	public double[] getMaxOfDataValues(){

		double[] ret = new double[this.dataAvg[0].length];

		Arrays.fill(ret, Double.NEGATIVE_INFINITY);

		for(int i = 0; i < this.dataAvg.length; i++){
			for(int j = 0; j < this.dataAvg[i].length; j++){
				ret[j] = Math.max(this.dataAvg[i][j], ret[j]);
			}
		}

		return ret;

	}

	/* returns an array that contains the values in the last row of the dataAvg matrix
	 **/
	public double[] getLastRowOfData(){

		double[] ret = new double[this.dataAvg[0].length];

		for(int i = 0; i < this.dataAvg[0].length; i++){
			ret[i] = this.dataAvg[this.dataAvg.length-1][i];
		}

		return ret;

	}

	public void printData(){

		System.out.println(DataCollector.getHeader());

		for(int i = 0; i < this.dataAvg.length; i++){
			System.out.println(DataCollector.getString(this.dataAvg[i]));
		}

	}

	public static String getHeader(){
		return "TIME_STEP   AVS_ALIVE   AVS_ON_AOIS   %AOIS_COVERED   AVG_AOIS_SEEN   AVG_%_TIME_ON_AOIS";
	}

	/* values contains one row from a dataAvg or dataStdDev matrix
	 **/
	public static String getString(double[] values){

		DecimalFormat df = new DecimalFormat("0.00000");
		String ret;

		ret = UtilityFunctions.leftPad((int)values[0], 6) + "      ";
		ret += UtilityFunctions.leftPad((int)values[1], 3) + "          ";
		ret += UtilityFunctions.leftPad((int)values[2], 3) + "           ";
		ret += UtilityFunctions.leftPad(df.format(values[3] * 100), 9) + "       ";
		ret += UtilityFunctions.leftPad(df.format(values[4]), 9) + "       ";
		ret += UtilityFunctions.leftPad(df.format(values[5] * 100.0), 9);

		return ret;

	}

}
