package sim.app.socialagents; 

import sim.util.Bag;
import ec.*; 
import ec.util.*; 
import ec.simple.*;


public class CommEvaluator extends Problem implements SimpleProblemForm { 
	
	public static final String P_NAME = "evaluator";
	public static final String P_RUNLENGTH = "runlength";
	public static final String P_DEPLOYMENT_INTERVAL = "deployment-interval"; 
	public static final String P_SENSOR_RADIUS = "sensor-radius"; 
	public static final String P_SPEED = "agent-speed"; 
	public static final String P_RECORD = "record-intervals"; 
	
	int runLength, recordIntervals;
	int width, height; 
	int numAgents; 
	int sensorRadius; 
	double agentSpeed; 
	//NonSocialAgentController controller; 
	SocialAgentController controller; 

	
	public void setup( final EvolutionState state, final Parameter base )
	{
		super.setup( state, base );
		runLength = state.parameters.getInt(base.push(P_RUNLENGTH),null,0); 
		recordIntervals = state.parameters.getInt(base.push(P_RECORD),null,0); 
		controller = new SocialAgentController(new MersenneTwisterFast());
		System.out.println(runLength);
	}
	
	public Parameter defaultBase()
	{
		return SocialDefaults.base().push(P_NAME); 
	}
		
	public void evaluate(final EvolutionState state,
                         final Individual ind,
                         final int threadnum) { 
		System.out.println("Evaluating fitness");
		
		if (ind.evaluated) return; 
		
		if (! (ind instanceof CommSocialIndividual)) 
			state.output.error("Whoa! Individual not instance of CommSocialIndividual");
		
		CommSocialIndividual ind2 = (CommSocialIndividual)ind;
		System.out.println(ind2);
		CommSocialSim sim = 
			new CommSocialSim(System.currentTimeMillis(), controller, 
					runLength, recordIntervals, "run.conf",ind2.commParams); 
		
		sim.start(); 
		long steps=0;
		CommSocialAgent scout = null; // this will be around scout to test the gradient
		while ((steps = sim.schedule.getSteps()) <= runLength){
			//state.output.message("Step: "+steps);
			if (! sim.schedule.step(sim)) break;
			if(scout != null && scout.reachedGoal){
				System.out.println("Scout found AOI");
				break;
			}

            if(steps==15000){
    			//	Testing gradients 
            	//System.out.println("Creating scout");
    			//Bag agents = new Bag(sim.agents.getAllObjects());
    			//scout = (CommSocialAgent)(agents.objs[10]);
    			//scout.designateAsSearchAgent();

    			scout = new CommSocialAgent(sim.agentInitVelocity,
    						sim.agentInitPosition, sim.agentSenseDist, sim.agentSize,
    						sim.agentSpeed, sim.random, sim.agentControl, sim);
    			scout.designateAsSearchAgent();

    			sim.agents.setObjectLocation(scout, sim.agentInitPosition);
    			sim.schedule.scheduleRepeating(sim.schedule.time()+1, scout);

    		}

		}
		
		
		//((SimpleFitness)ind.fitness).setFitness(state,(float)sim.obtainFitnessForGA(), false); 
		System.out.println(sim.schedule.getSteps()-1);
		System.out.println(((float)(runLength - (sim.schedule.getSteps()-1)))/runLength);
		((SimpleFitness)ind.fitness).setFitness(state,((float)(runLength - (sim.schedule.getSteps()-1)))/runLength, false); 
		
		sim.finish();

		//clean up
		sim=null;
		scout=null;
		
		ind2.evaluated = true; 
	}
	
	
	public void describe(final Individual ind, 
                         final EvolutionState state, 
                         final int threadnum, final int log,
                         final int verbosity)    { }
	
}