package sim.app.socialagents;

import sim.engine.*;
import sim.portrayal.*;
import sim.util.*;

import java.awt.*;
import java.util.HashMap;
import java.util.Map;

import bioutil.CommCreateGradientThread;
import bioutil.CommReceiveGradientThread;
import bioutil.Morphogen;
import bioutil.Message;
import bioutil.SearchState;
import ec.util.*;

import java.util.LinkedList;
import java.lang.Runnable;
import java.util.concurrent.*;
import java.util.Queue;
import java.util.ArrayList;

/**
 * @author Adelein Rodriguez
 */

public class CommSocialAgent extends CommAgent {

	public boolean debugCommunication = false;

	public static int GlobalId = 0;

	public int id;

	private MutableDouble2D currentGoal;

	private MutableDouble2D goalInMemory;

	public final CommSocialSim state;

	private MersenneTwisterFast rng;

	// Gradient information
	public Map morphogenList; // list of morphogens seen so far

	private boolean source; // whether or not agent is a source of a gradient

	public static Map messages;

	ExecutorService exec;

	CommReceiveGradientThread commReceive;

	CommCreateGradientThread commCreate;

	// These boolean will be used to try to stop
	// the send and receive threads for the scout
	public boolean stopReceive = false;

	public boolean stopCreate = false;

	boolean hasGradientInfo; // if agent is propagating a gradient right now

	public Morphogen activeGradient;

	public boolean searchingMode; // If agent is searching for a source

	public CommSocialAgent previousNeighbor;

	public int faithCount; // num steps it has gone in one direction without
							// finding
	// another agent
	public int faithMax = 200; // max number of vecloty update steps (not
								// necessarily steps)
	// it cant go without finding another neighbor
	// before it reverses its direction

	public boolean hasSeenAtLeastOneGradient = false;

	public int timeOnGuidedByPreviousBNeighbor = 0; // how long will we follow
	// the same best neighbor

	// while on search mode.
	public int maxFanaticTime = 5; // Max number of a times an agent can follow

	// the same other agent while on search mode.
	boolean hasGoal;

	// Params that will be evolved
	public CommunicationParameters commParams;

	public SearchState searchState;

	public double previousAvgGradient;

	public boolean reachedGoal;

	public int bestSeenMorphogenSoFar = -1;

	public void designateAsSearchAgent() {
		System.out.println("Designating search agent");
		this.searchState = new SearchState(this, this.activeGradient);
		searchingMode = true;
		AgentController newController = this.getController();
		newController.changeParameters(new double[] { 0.01, 0.88162, 0.0, 1.0,
				0.5, 0.13847, 0.0, 0.0 });
		this.setController(newController);
		this.maxSpeed = state.getAgentSpeed();
		this.sensorDist += 100;
		this.diameter += 200;

		// Use customized parameters
		double[] params = new double[5];
		params[0] = 100;
		params[1] = 1000;
		params[2] = 1000;
		params[3] = 40;
		params[4] = 3.0;

		this.commParams = new CommunicationParameters(params);
		killThreads();
	}

	public void killThreads() {
		this.stopCreate = true;
		this.stopReceive = true;
		if (this.commReceive != null) {
			this.commReceive.stop = true;
		}
		if (this.commCreate != null) {
			this.commCreate.stop = true;
		}

		// join the thread.
		try {
			if (this.exec != null) {
				// we need to do a spin-wait interrupt, then join; rather than
				// a single interrupt followed by a join, because it's possible
				// that the play thread could test if it's interrupted, see that
				// it's not, then we interrupt it, and THEN the play thread goes
				// into its blocking situation. This is extremely unlikely but
				// theoretically possible. So we repeatedly interrupt the thread
				// even in this situation until it gets a clue.
				// do
				// {
				// try
				// {
				// grab lock on schedule so interruption can't
				// occur within movie-making (which causes JMF to freak out)
				// I hope this doesn't mess up things. Looks like it shouldn't
				// (the function of interrupt() here is to release
				// invokeAndWait,
				// and at that point nothing's blocked on the schedule)
				// synchronized(simulation.state.schedule)
				// {
				this.exec.shutdown();
				// }
				// }
				// catch (SecurityException ex) { } // some stupid browsers --
				// *cough* IE -- don't like interrupts
				this.exec.shutdownNow();
				// }
				// while(!this.exec.isTerminated());
				if (this.exec.isTerminated()) {
					commCreate = null;
					commReceive = null;
					System.out.println("Killed threads");
				}
				this.exec = null;
			}
		} catch (Exception e) {
			System.out.println("This should never happen: " + e);
		}
	}

	public void finalize() {
		killThreads();
	}

	public CommSocialAgent(Double2D vel, Double2D pos, double sense,
			int agentSize, double speed, MersenneTwisterFast rng,
			AgentController control, CommSocialSim state) {

		super(vel, pos, agentSize, sense, new Color(0, 0, 0), speed);

		this.id = GlobalId;
		GlobalId++;

		this.currentGoal = null;
		this.goalInMemory = null;
		this.rng = rng;

		/*
		 * this is extremely ugly, but this is how the agent's control system is
		 * initialized when the GA is being run
		 */
		this.controller = control;
		this.state = state;
		/*
		 * if no GA is running, a default control system is loaded
		 */
		if (this.controller == null) {

			// gradientDuration=100;
			// propagationDelay=1000;
			// checkIfMessageDelay=1000;
			// gradientStrength=40;
			// speedWhilePropagating=0.05;
			double[] params = new double[5];
			params[0] = 921;// 1000;
			params[1] = 402;// 1000;
			params[2] = 224;// 224;//100;
			params[3] = 58;
			params[4] = 0.005;// 0.01;
			// 921.69370 402.95592 224.70101 58.96740 0.05520

			this.commParams = new CommunicationParameters(params);

		} else {
			this.commParams = new CommunicationParameters(state.commParams);
		}
		this.controller = new SocialAgentController(rng);

		boolean[] cond = new boolean[AgentController.NUM_CONDITIONS];
		double[] parameters = new double[AgentController.NUM_PARAMS];

		parameters = new double[] { 0.001515, 0.88162, 0.67661, 1.0, 0.65,
				0.13847, 0.22561, 0.0 };
		this.controller.addControl(cond, parameters);

		// init gradient parameters
		morphogenList = new HashMap();
		messages = new HashMap();
		source = false;
		exec = Executors.newCachedThreadPool();
		commReceive = new CommReceiveGradientThread(this);
		exec.execute(commReceive);
		hasGradientInfo = false;
		hasGoal = false;
		previousNeighbor = null;
		activeGradient = null;
	}

	public void step(SimState state) {

		// if the agent is dead, we can exit out of this function
		if (!super.isAlive())
			return;

		CommSocialSim sim = (CommSocialSim) state;
		MutableDouble2D newVel = null;

		// the following is for the path drawing only record the Agent's
		// position for the path each 25 time steps

		if (sim.schedule.getSteps() % 25 == 0) {

			this.path[this.pathIndex % this.maxPathLength] = new Double2D(
					this.positionx, this.positiony);
			this.pathIndex++;
		}

		// get the positions of nearby agents, obstacles and AOIs
		this.getNearbyAgentPositions(state);
		this.getNearbyObstaclePositions(state);
		this.getNearbyAOIPositions(state);

		this.activated = true;

		// reverse direction if a wall is encountered

		if ((this.positionx - this.radius <= 0.0 && this.velocityx < 0.0)
				|| (this.positionx + this.radius >= sim.fieldWidth && this.velocityx > 0.0)) {
			this.velocityx *= -1.0;
		}

		if ((this.positiony - this.radius <= 0.0 && this.velocityy < 0.0)
				|| (this.positiony + this.radius >= sim.fieldHeight && this.velocityy > 0.0)) {
			this.velocityy *= -1.0;
		}

		this.obtainMemory(state);
		// this.sendMessage(state);

		// if it is time to update the agent, then... update the agent's comfort
		// zone obtain the Agents' new velocity from its controller

		// the controller takes into account the agent's internal state in order
		// to figure out which set of control parameters it should use

		if (sim.schedule.getSteps() % this.updateVelocityInterval == 0) {

			// So every two steps we are going to try to compute a
			// new direction if the agent is in searching mode.
			// If the agent has no neighbors to guide him then
			// the agent will just use the default moving behavior
			// if(sim.schedule.getSteps() % 2 == 0){
			this.comfortZone = this.controller.getNonGoalParam(this,
					AgentController.NG_COMFORT);

			newVel = this.controller.getNewVelocity(this);
			if (this.searchingMode && sim.schedule.getSteps() % 12 == 0) {
				this.maxSpeed = this.state.getAgentSpeed();
				MutableDouble2D followVel;
				if ((followVel = this.getNewGradientDirection()) != null) {

					// this.setHighlight(true);

					hasGoal = true;
					newVel = new MutableDouble2D(0, 0);
					newVel.addIn(followVel);
				}
			}

			if (newVel.lengthSq() != 0.0) {
				newVel.normalize();
				newVel.multiplyIn(this.maxSpeed);
			}

			this.updateVelocity(newVel.x, newVel.y);

		}
		// If we have seen an AOI then let others know
		if (this.getOnAOI() && !this.source && !this.searchingMode) {
			String gradientName = this + "";
			this.createGradient(new Morphogen(gradientName,
					(int) this.commParams.getGradientStrength()));

		}
		if (this.searchingMode && this.getOnAOI()) {
			reachedGoal = true;
		}
		this.moveOneStep(state);

	}

	public void moveOneStep(SimState state) {

		CommSocialSim sim = (CommSocialSim) state;

		/*
		 * update the position in the Continuous2D container also, update the
		 * position within the agent
		 */
		Double2D newpos = new Double2D(this.positionx + this.velocityx,
				this.positiony + this.velocityy);
		sim.agents.setObjectLocation(this, newpos);

		this.updatePosition(newpos);

	}
	

	public MutableDouble2D getVectorToNeighbor(CommSocialAgent agent) {

		double comfortZoneSq = (this.sensorDist * this.sensorDist)
				* (this.comfortZone * this.comfortZone);
		MutableDouble2D thisPosition = this.getPosition();
		MutableDouble2D neighborPosition = new MutableDouble2D(agent
				.getPosition().x, agent.getPosition().y);

		MutableDouble2D closestPoint;
		closestPoint = new MutableDouble2D(Double.MAX_VALUE, Double.MAX_VALUE);
		Double2D obstaclePosition;
		double minDist = Double.MAX_VALUE;

		MutableDouble2D vector = new MutableDouble2D(0.0, 0.0);

		// Find the closest obstacle point

		for (int i = 0; i < this.obstaclePositions.numObjs; i++) {

			obstaclePosition = (Double2D) this.obstaclePositions.objs[i];

			if (thisPosition.distanceSq(neighborPosition) < minDist) {
				closestPoint = new MutableDouble2D(neighborPosition);
				minDist = thisPosition.distanceSq(neighborPosition);
			}
		}
		System.out.println("Number of obstacles: "
				+ this.obstaclePositions.numObjs);

		double tmpx, tmpy, magn;
		tmpx = neighborPosition.x - this.positionx;
		tmpy = neighborPosition.y - this.positiony;
		magn = Math.sqrt((tmpx * tmpx) + (tmpy * tmpy));

		if (magn != 0.0) {
			tmpx /= magn;
			tmpy /= magn;

			vector.addIn(tmpx, tmpy);
		}

		if (vector.lengthSq() != 0.0) {
			vector.normalize();
		}

		return vector;
	}

	public MutableDouble2D getRandomVelocity() {

		double randomx, randomy;
		MutableDouble2D randomVector = new MutableDouble2D();

		randomx = this.rng.nextDouble()
				* ((this.rng.nextInt(2) == 0) ? 1.0 : -1.0);
		randomy = this.rng.nextDouble()
				* ((this.rng.nextInt(2) == 0) ? 1.0 : -1.0);
		randomVector = new MutableDouble2D(randomx, randomy);

		/*
		 * if(randomVector.lengthSq() != 0.0){ randomVector.normalize();
		 * randomVector
		 * .multiplyIn(this.nonGoalParams[curConditionIndex][AgentController
		 * .NG_RANDOMNESS]); }
		 */

		return randomVector;
	}

	/*
	 * Returns a velocity vector to a nearby agent if this agent is in search
	 * mode. Otherwise it returns null.
	 */
	public MutableDouble2D getNewGoal() {

		MutableDouble2D newVel = null;
		CommSocialAgent bestNeighbor;
		// If the agent is in search mode because through the higher level
		// simulation we told it to be, then we also need to make sure that
		// the agent is in two possible states, with or without a gradient
		// value itself. If it has no gradient value it is equivalent to
		// having a worse value that its neighbors.
		if (searchingMode) {
			System.out.println(searchState.getSearchGuideGradient());
			bestNeighbor = this.getHighestGradientNeighbor((searchState
					.getSearchGuideGradient() != null ? searchState
					.getSearchGuideGradient().name : null));

			if (bestNeighbor != null) {
				/*
				 * // If the current best neighbor has a value better than mine
				 * // or if I have no value and my neighbor does // then compute
				 * velocity vector toward the neighbor
				 * 
				 * if((bestNeighbor.activeGradient!=null &&
				 * this.activeGradient!=null &&
				 * bestNeighbor.activeGradient.value >
				 * this.activeGradient.value) || (this.activeGradient==null &&
				 * bestNeighbor.activeGradient!=null )){
				 * 
				 * newVel=getVectorToGoal(bestNeighbor); // We are not using the
				 * neighbor so // we need to find out whether it was the same as
				 * before. // If so then increase the count;
				 * if(sameAsPreviousMarker(bestNeighbor)){
				 * System.out.println("Prev: "+this.previousNeighbor+ " Curr:
				 * "+bestNeighbor+" Increasing time from
				 * "+this.timeOnGuidedByPreviousBNeighbor);
				 * this.timeOnGuidedByPreviousBNeighbor++; }else{ // We have a
				 * new guide System.out.println("Prev: "+this.previousNeighbor+
				 * " Curr: "+bestNeighbor+" Resetting time from
				 * "+this.timeOnGuidedByPreviousBNeighbor);
				 * this.previousNeighbor=bestNeighbor;
				 * this.timeOnGuidedByPreviousBNeighbor=0; }
				 * updateColor(bestNeighbor.activeGradient.value);
				 * 
				 * }//end else
				 */
				if (this.searchState.okToFollowNeighbhor(bestNeighbor)) {
					newVel = getVectorToNeighbor(bestNeighbor);
					updateColor(this.searchState.getSearchGuideGradient().value);
				}

			}
		}

		return newVel;

	}

	public MutableDouble2D getNewGradientDirection() {

		MutableDouble2D newVel = null;
		// If the agent is in search mode because through the higher level
		// simulation we told it to be, then we also need to make sure that
		// the agent is in two possible states, with or without a gradient
		// value itself. If it has no gradient value it is equivalent to
		// having a worse value that its neighbors.
		if (searchingMode) {
			// System.out.println(searchState.getSearchGuideGradient());
			newVel = this.getAnyAgentGradientDirectionVector();
			if (newVel != null) {
				// System.out.println("Updated Velocity: "+newVel);
				if (this.activeGradient != null) {
					System.out.println(this.activeGradient.value);
					updateColor(this.activeGradient.value);
				} else {
					updateColor(1);
				}

			}
		}

		return newVel;

	}

	public MutableDouble2D getGradientDirectionVector() {
		ArrayList<CommSocialAgent> neighbors = new ArrayList<CommSocialAgent>();
		neighbors = this.getNeighbors();
		MutableDouble2D newVel = new MutableDouble2D(0, 0);
		Morphogen bestNeighborGradient = new Morphogen("", 0);
		if (neighbors.size() == 0) {
			Morphogen currentGradient = this.searchState
					.getSearchGuideGradient();
			if (currentGradient != null) {
				Morphogen fadedMorphogen = new Morphogen(currentGradient.name,
						currentGradient.value);
				int oldMorphogenValue = fadedMorphogen.value;
				fadedMorphogen.value -= 5;

				if (fadedMorphogen.value <= -100
						&& this.hasSeenAtLeastOneGradient) {
					this.activeGradient = null;
					this.searchState.setSearchGuideGradient(null);
					return new MutableDouble2D(this.velocityx * -1,
							this.velocityy * -1);
				} else {
					this.activeGradient = fadedMorphogen;
					this.searchState.setSearchGuideGradient(fadedMorphogen);
				}
			}
			newVel = null;

		} else {

			double averageGradient = 0;
			int numNeighborsWithGradient = 0;
			CommSocialAgent bestNeighbor = this.previousNeighbor;// dummy value
			for (CommSocialAgent a : neighbors) {

				if (a.alive && a.activeGradient != null) {
					this.hasSeenAtLeastOneGradient = true;
					// Compute Velocity vector to neighbor
					MutableDouble2D vector = new MutableDouble2D();

					vector.subtract(this.getPosition(), a.getPosition());
					vector.x *= -1;
					vector.y *= -1;

					// Depending on their gradient value the velocity
					// vector gets a higher magnitude

					vector.multiplyIn(a.activeGradient.value);
					if (vector.lengthSq() != 0.0) {
						vector.normalize();
					}
					averageGradient += a.activeGradient.value;
					numNeighborsWithGradient++;
					if (a.activeGradient.value > bestNeighborGradient.value) {
						bestNeighborGradient = a.activeGradient;
						bestNeighbor = a;
					}

					newVel.addIn(vector);

				}
			}

			averageGradient /= numNeighborsWithGradient;
			// If the previous avg gradient was best than this one or
			// If we have already seen the neighbor with the highest gradient
			// here
			// do not use the new velocity
			if (averageGradient < this.previousAvgGradient
					|| (this.previousNeighbor != null && bestNeighbor.id == this.previousNeighbor.id)) {
				newVel = null;

			}

			this.previousAvgGradient = averageGradient;
			// However, because we have seen some new gradient information
			// we need to save it whether we use it to modify our velocty or
			// not.
			// So our velocity will be in the direction of the average gradient
			// but our reference gradient will be the best from our neighbors
			// at any point

			this.activeGradient = bestNeighborGradient;
			this.searchState.setSearchGuideGradient(activeGradient);

			// We need to save the best neighbor anyways as the previous
			// We could also have saved any other agent as a marker
			// that the agent was in this place already
			this.previousNeighbor = bestNeighbor;

		}
		/*
		 * if(this.bestSeenMorphogenSoFar < bestNeighborGradient.value &&
		 * this.bestSeenMorphogenSoFar!=-1){
		 * this.bestSeenMorphogenSoFar=bestNeighborGradient.value;
		 * this.faithCount=0; // Reset because we have improved }else{ // Best
		 * neighbor did not improve, maybe we are moving // in the wrong
		 * direction? this.faithCount++; if(this.faithCount > this.faithMax){ //
		 * Reverse Direction newVel=new
		 * MutableDouble2D(this.velocityx*-1,this.velocityy*-1);
		 * this.faithCount=0; }
		 * 
		 * }
		 */

		return newVel;
	}

	public MutableDouble2D getAnyAgentGradientDirectionVector() {
		ArrayList<CommSocialAgent> neighbors = new ArrayList<CommSocialAgent>();
		neighbors = this.getNeighbors();
		Morphogen bestNeighborGradient = new Morphogen("", 0);
		MutableDouble2D newVel = new MutableDouble2D();
		MutableDouble2D vectorToBest = new MutableDouble2D();
		if (neighbors.size() == 0) {

			return null;

		} else {

			double averageGradient = 0;
			int numNeighborsWithGradient = 0;
			CommSocialAgent bestNeighbor = neighbors.get(0);// dummy value
			int bestNeighborGradientValue = (bestNeighbor.activeGradient != null ? bestNeighbor.activeGradient.value
					: -1);

			for (int x = 1; x < neighbors.size(); x++) {
				CommSocialAgent a = neighbors.get(x);
				if (a.alive && a.activeGradient != null) {
					this.hasSeenAtLeastOneGradient = true;
					// Compute Velocity vector to neighbor
					MutableDouble2D vector = new MutableDouble2D();

					vector.subtract(this.getPosition(), a.getPosition());
					vector.x *= -1;
					vector.y *= -1;

					// Depending on their gradient value the velocity
					// vector gets a higher magnitude

					vector.multiplyIn(a.activeGradient.value);
					if (vector.lengthSq() != 0.0) {
						vector.normalize();
					}
					averageGradient += a.activeGradient.value;
					numNeighborsWithGradient++;
					if (bestNeighborGradientValue <= a.activeGradient.value) {
						bestNeighborGradientValue = a.activeGradient.value;
						vectorToBest = vector;
						bestNeighbor = a;
					}
					newVel.addIn(vector);

				}
			}
			if (vectorToBest.x == 0 && vectorToBest.y == 0) {
				return null;
			}
			// this.activeGradient=new
			// Morphogen(bestNeighbor.activeGradient.name,bestNeighbor.activeGradient.value);
			// this.updateColor(this.activeGradient.value);

		}

		return vectorToBest;
	}

	public boolean betterThan(CommSocialAgent other) {

		if (other == null || other.activeGradient == null) {
			return false;
		}
		if (this.activeGradient == null) {
			return true;

		} else if (other.activeGradient.value >= this.activeGradient.value) {
			return true;
		}

		return false;
	}

	// sets goalInMemory if the agent is on top of an AOI
	// also removes the agents' current goal if the agent is on the current goal
	// AOI

	private Bag nearbyAOIs;

	private void obtainMemory(SimState state) {

		if (this.getOnAOI()) {

			CommSocialSim sim = (CommSocialSim) state;
			nearbyAOIs = new Bag();

			sim.areasOfInterest.getObjectsWithinDistance(new Double2D(
					this.positionx, this.positiony), this.getSensorDist(),
					false, false, nearbyAOIs);

			for (int i = 0; i < nearbyAOIs.numObjs; i++) {

				final AreaOfInterest aoi = (AreaOfInterest) (nearbyAOIs.objs[i]);

				// skip the region if it is not active
				if (!aoi.isActive()) {
					continue;
				}

				/*
				 * set the goal in the agent's memory to the AOI's center
				 */
				if (aoi.pointInsideRegion(this.positionx, this.positiony)) {

					// remove the current goal
					if (this.currentGoal != null
							&& this.currentGoal.equals(aoi.getCenter()))
						this.currentGoal = null;

					this.goalInMemory = new MutableDouble2D(aoi.getCenter());
					break;

				}

			}

		}

	}

	// sends the goalInMemory to nearby agents
	// private Bag nearbyAgents;
	/*
	 * private void sendMessage(SimState state){
	 * 
	 * if(this.goalInMemory != null){
	 * 
	 * CommSocialSim sim = (CommSocialSim)state; Bag nearbyAgents = new Bag();
	 * 
	 * sim.agents.getObjectsWithinDistance(new
	 * Double2D(this.positionx,this.positiony
	 * ),this.getSensorDist(),false,false,nearbyAgents); // iterate through each
	 * nearby agent and check whether or not it is within sensor range of this
	 * for(int i = 0; i < nearbyAgents.numObjs; i++){
	 * 
	 * if(!(nearbyAgents.objs[i] instanceof CommSocialAgent)) continue;
	 * 
	 * final CommSocialAgent otherAgent =
	 * (CommSocialAgent)(nearbyAgents.objs[i]); final double dist =
	 * Math.sqrt((otherAgent
	 * .positionx-this.positionx)*(otherAgent.positionx-this
	 * .positionx)+(otherAgent
	 * .positiony-this.positiony)*(otherAgent.positiony-this.positiony)); //
	 * send the message to the nearby agent if(dist < this.sensorDist +
	 * otherAgent.radius && otherAgent.isAlive() && otherAgent.isActivated() &&
	 * dist != 0.0){ otherAgent.receiveMessage(this.goalInMemory); } } } }
	 * 
	 * public void receiveMessage(MutableDouble2D message){
	 * 
	 * if(this.goalInMemory == null || !this.goalInMemory.equals(message)){
	 * this.currentGoal = new MutableDouble2D(message); } }
	 */

	public MutableDouble2D getGoalPoint() {
		return this.currentGoal;
	}

	public MutableDouble2D getMemory() {
		return this.goalInMemory;
	}

	// This function should consider neighbors within a
	// sensor distance. The idea
	// is that we want to simulate an agent broadcasting a message
	// to a medium, rather than to other agents specifically and directly.
	// So neighbors are those agents for which this agent can see
	// their sensor ranges at least. The assumption is that sensor range is
	// equivalent to the area affected by the agent's gradient.
	// Sensor range is the same as the effector range.
	// Thus it is sufficient that an agent can see this are to know the
	// gradient emitted by the other. We also assume that all agents have
	// the same sensor range
	public ArrayList getNeighbors() {

		Bag nearbyAgents = new Bag();

		this.state.agents.getObjectsWithinDistance(new Double2D(this.positionx,
				this.positiony), (this.getSensorDist() * 2) - 1, false, false,
				nearbyAgents);
		ArrayList neighbors = new ArrayList<CommSocialAgent>();
		for (int i = 0; i < nearbyAgents.numObjs; i++) {

			if (!(nearbyAgents.objs[i] instanceof CommSocialAgent)) {
				continue;
			}

			CommSocialAgent otherAgent = (CommSocialAgent) (nearbyAgents.objs[i]);
			double distToNeighbor = this.getPosition().distance(
					otherAgent.getPosition());
			// Because getObjectsWithinDistance considers a squared area around
			// the agent, and we really want a circle area, we throw away agents
			// in the corners of the square.Also make sure we d onot return this
			// agent
			// as a neighbor of itself, duh.
			if (otherAgent.id == this.id
					|| distToNeighbor > this.getSensorDist() * 2) {
				continue;
			}

			neighbors.add(otherAgent);

		}
		return neighbors;

	}

	// ----------These are the functions needed to do neighborhood
	// querying----------//

	/*
	 * public ArrayList getCloseNeighbors() {
	 * 
	 * Bag nearbyAgents = new Bag();
	 * 
	 * this.state.agents.getObjectsWithinDistance(new Double2D(this.positionx,
	 * this.positiony), (this.getSensorDist()*2)-1, false, false, nearbyAgents);
	 * ArrayList neighbors = new ArrayList<CommSocialAgent>(); for (int i = 0; i
	 * < nearbyAgents.numObjs; i++) {
	 * 
	 * if (!(nearbyAgents.objs[i] instanceof CommSocialAgent) ||
	 * nearbyAgents.objs[i] == this) continue; CommSocialAgent otherAgent =
	 * (CommSocialAgent) (nearbyAgents.objs[i]); final double dist = Math
	 * .sqrt((otherAgent.positionx - this.positionx) (otherAgent.positionx -
	 * this.positionx) + (otherAgent.positiony - this.positiony)
	 * (otherAgent.positiony - this.positiony)); // check to see if otherAgent
	 * is within sensor distance // if(dist < this.sensorDist +
	 * otherAgent.radius && // otherAgent.isAlive() && dist != 0.0){
	 * neighbors.add(otherAgent); // }
	 * 
	 * } return neighbors;
	 * 
	 * }
	 */

	// This function should consider neighbors within sensor range only.
	// An agent within another agent's sensor range is not a close neighbor
	/*
	 * public ArrayList getCloseNeighborsWithGradient() {
	 * 
	 * Bag nearbyAgents = new Bag();
	 * 
	 * this.state.agents.getObjectsWithinDistance(new Double2D(this.positionx,
	 * this.positiony), (this.getSensorDist()*2)-1, false, false, nearbyAgents);
	 * ArrayList neighbors = new ArrayList<CommSocialAgent>(); for (int i = 0; i
	 * < nearbyAgents.numObjs; i++) {
	 * 
	 * if (!(nearbyAgents.objs[i] instanceof CommSocialAgent) ||
	 * nearbyAgents.objs[i] == this) continue; CommSocialAgent otherAgent =
	 * (CommSocialAgent) (nearbyAgents.objs[i]); if (otherAgent.activeGradient
	 * != null) { neighbors.add(otherAgent); } } return neighbors;
	 * 
	 * }
	 */

	// This function should consider neighbors within sensor range only.
	// An agent within another agent's sensor range is not a close neighbor
	/*
	 * public double getNeighborsAverageGradient(String gradientName){
	 * 
	 * double neighborsSum=0; ArrayList <CommSocialAgent>neighbors = new
	 * ArrayList<CommSocialAgent>(); neighbors=this.getCloseNeighbors();
	 * for(CommSocialAgent a: neighbors){
	 * 
	 * neighborsSum=+((Morphogen)a.morphogenList.get(gradientName)).value; }
	 * 
	 * return neighborsSum/nearbyAgents.numObjs; }
	 */

	/*
	 * This function returns the neighbhor with highest gradient value.
	 */
	// TODO need to fix this so that it goes through nearby agents loop once
	// only
	// to do everything as opposed to calling getNeighbors and the doing the
	// other
	public CommSocialAgent getHighestGradientNeighbor(String gradientName) {

		ArrayList<CommSocialAgent> neighbors;// = new
												// ArrayList<CommSocialAgent>();
		neighbors = this.getNeighbors();
		int numNeighbors = 0;
		CommSocialAgent best = null;
		// Here we are going to get the neighbor
		// with the highest gradient value. Regular
		// findMax algorithm.
		if (neighbors.size() > 0) {
			best = (CommSocialAgent) neighbors.get(0);
			numNeighbors++;
			for (CommSocialAgent a : neighbors) {
				Morphogen otherAgentMorphogen;
				// If this agent has gradient info at all
				if (a.getActiveMorphogen() != null
						&& best.getActiveMorphogen() != null) {
					otherAgentMorphogen = a.getActiveMorphogen();
					// if this gradient value is better
					numNeighbors++;
					if (((Morphogen) best.morphogenList.get(gradientName)).value > otherAgentMorphogen.value) {
						best = a;
					}// end if
				}// end if
			}// end for
			if (best.morphogenList.get(gradientName) != null) {
				return best;
			} else {
				return null;
			}
		} else {
			return null;
		}

	}

	// This function should consider neighbors within sensor range only.
	// An agent within another agent's sensor range is not a close neighbor
	public int getNumNeighbors(SimState state) {
		return this.getNeighbors().size();
	}

	public void draw(Object object, Graphics2D graphics, DrawInfo2D info) {
		// values for the circle
		final double width = info.draw.width * this.diameter;
		final double height = info.draw.height * this.diameter;
		final int x = (int) (info.draw.x - width / 2.0);
		final int y = (int) (info.draw.y - height / 2.0);
		final int w = (int) (width);
		final int h = (int) (height);

		if (this.alive) {

			// values for the direction
			final double magn = Math.sqrt(this.velocityx * this.velocityx
					+ this.velocityy * this.velocityy);
			final int xline, yline;

			if (magn != 0.0) {
				xline = (int) (info.draw.x + (velocityx / magn)
						* (info.draw.width * this.radius));
				yline = (int) (info.draw.y + (velocityy / magn)
						* (info.draw.height * this.radius));
			} else {
				xline = (int) (info.draw.x);
				yline = (int) (info.draw.y);
			}

			// values for the sensor
			final double sensorWidth = info.draw.width * this.sensorDist * 2.0;
			final double sensorHeight = info.draw.height * this.sensorDist
					* 2.0;
			final int sensorX = (int) (info.draw.x - sensorWidth / 2.0);
			final int sensorY = (int) (info.draw.y - sensorHeight / 2.0);
			final int sensorW = (int) (sensorWidth);
			final int sensorH = (int) (sensorHeight);

			// draw the sensor circle
			graphics.setColor(new Color(63, 63, 63));
			graphics.drawOval(sensorX, sensorY, sensorW, sensorH);

			// draw the AV centered on the origin
			if (this.highlighted) {
				graphics.setColor(this.highlightColor);
			} else {
				graphics.setColor(this.color);
			}
			graphics.fillOval(x, y, w, h);
			graphics.setColor(Color.blue);
			graphics.drawString(this.id
					+ " "
					+ (this.getActiveMorphogen() != null ? this
							.getActiveMorphogen().value : -1), x, y);
			if (this.searchingMode) {
				// draw the velocity
				graphics.setColor(Color.black);
				graphics.drawString("(" + this.velocityx + "," + this.velocityy
						+ ")", x, y - 10);
			}

			// draw its path
			if (this.drawPath && this.pathIndex >= 2) {

				int x1, y1, x2, y2;

				if (this.pathIndex < this.maxPathLength) {
					for (int i = 0; i < this.pathIndex - 1; i++) {
						graphics.setColor(new Color(
								255 - (int) (255.0 / this.pathIndex * i),
								255 - (int) (255.0 / this.pathIndex * i),
								255 - (int) (255.0 / this.pathIndex * i)));
						x1 = (int) (info.draw.width * this.path[i].x);
						y1 = (int) (info.draw.height * this.path[i].y);
						x2 = (int) (info.draw.width * this.path[i + 1].x);
						y2 = (int) (info.draw.height * this.path[i + 1].y);
						graphics.drawLine(x1, y1, x2, y2);
					}
				} else {
					/*
					 * for(int i = this.pathIndex, j = 0; i < (this.pathIndex +
					 * this.maxPathLength) - 1; i++, j++){ graphics.setColor(new
					 * Color(255 - (int)(255.0/this.maxPathLength * j), 255 -
					 * (int)(255.0/this.pathIndex * i), 255 -
					 * (int)(255.0/this.pathIndex * i))); x1 =
					 * (int)(info.draw.width * this.path[i %
					 * this.maxPathLength].x); y1 = (int)(info.draw.height *
					 * this.path[i % this.maxPathLength].y); x2 =
					 * (int)(info.draw.width * this.path[(i+1) %
					 * this.maxPathLength].x); y2 = (int)(info.draw.height *
					 * this.path[(i+1) % this.maxPathLength].y);
					 * graphics.drawLine(x1, y1, x2, y2); }
					 */
				}

			}

			// draw lines to nearby agents
			graphics.setColor(Color.red);
			if (this.agentPositions != null) {
				Double2D otherAgentPosition;
				for (int i = 0; i < this.agentPositions.numObjs; i++) {
					otherAgentPosition = (Double2D) this.agentPositions.objs[i];
					graphics.drawLine((int) info.draw.x, (int) info.draw.y,
							(int) (info.draw.width * otherAgentPosition.x),
							(int) (info.draw.height * otherAgentPosition.y));
				}
			}

			// draw lines to nearby obstacles
			graphics.setColor(Color.orange);
			if (this.obstaclePositions != null) {
				Double2D obstaclePos;
				for (int i = 0; i < this.obstaclePositions.numObjs; i++) {
					obstaclePos = (Double2D) (this.obstaclePositions.objs[i]);
					graphics.drawLine((int) info.draw.x, (int) info.draw.y,
							(int) (info.draw.width * obstaclePos.x),
							(int) (info.draw.height * obstaclePos.y));
				}
			}

			// draw its direction
			graphics.setColor(Color.white);
			graphics.drawLine((int) info.draw.x, (int) info.draw.y, xline,
					yline);

			// values for the gradient direction
			final int gxline, gyline;
			MutableDouble2D gvelocity = this
					.getAnyAgentGradientDirectionVector();
			if (gvelocity != null && gvelocity.x != 0 && gvelocity.y != 0) {
				final double gmagn = Math.sqrt(gvelocity.x * this.velocityx
						+ gvelocity.y * gvelocity.y);

				if (magn != 0.0) {
					gxline = (int) (info.draw.x + (gvelocity.x / gmagn)
							* (info.draw.width * this.radius));
					gyline = (int) (info.draw.y + (gvelocity.y / gmagn)
							* (info.draw.height * this.radius));
				} else {
					gxline = (int) (info.draw.x);
					gyline = (int) (info.draw.y);
				}

				graphics.setColor(Color.BLUE);
				graphics.drawLine((int) info.draw.x, (int) info.draw.y, gxline,
						gyline);

			}

		} else {

			graphics.setColor(Color.black);
			graphics.fillOval(x, y, w, h);

		}

	}

	public Morphogen getActiveMorphogen() {
		if (searchState != null) {
			return searchState.getSearchGuideGradient();
		} else {
			return this.activeGradient;
		}
	}

	public void updateActiveMorphogen(Morphogen morphogen) {
		this.activeGradient = morphogen;
	}

	public void updateColor(int morphogenValue) {
		if (morphogenValue <= 40) {
			this.setColor(new Color(5 * morphogenValue, 100, 100));
		} else if (morphogenValue <= 100) {
			this.setColor(new Color(2 * morphogenValue, 100, 100));
		}
	}

	// returns true if it will be able to propagate the signal
	public boolean receiveGradient(Morphogen morphogen) {
		if (debugCommunication) {
			System.out.println(this.id + " Receiving signal " + morphogen.name
					+ " " + morphogen.value);
		}
		Morphogen existingMorphogen = (Morphogen) morphogenList
				.get(morphogen.name);
		// if this morphogen has not been seen before or if it has been seen
		// but its value is greater than an existing morphogen saved value then
		// save it
		if (existingMorphogen == null
				|| (existingMorphogen != null && existingMorphogen.value < morphogen.value)
				&& !this.searchingMode) {

			morphogenList.put(morphogen.name, morphogen);
			activeGradient = morphogen;
			if (debugCommunication) {
				System.out.println(this.id + " Saving " + morphogen.name + " "
						+ morphogen.value);
			}
			hasGradientInfo = true;
			if (!this.source) {
				// System.out.println(morphogen.value);
				updateColor(morphogen.value);
				this.maxSpeed = this.commParams.getSpeedWhilePropagating();
			}
			if (morphogen.value == 0)
				return false;
			else
				return true;
		} else {
			if (debugCommunication) {
				System.out.println(this.id + " Ignoring gradient");
			}
			return false;
		}

	}

	public void createGradient(Morphogen morphogen) {
		if (debugCommunication) {
			System.out.println(this.id + " Creating gradient");
		}
		source = true;
		morphogenList.put(morphogen.name, morphogen);
		this.activeGradient = morphogen;
		this.setHighlight(true);
		this.maxSpeed = this.commParams.getSpeedWhilePropagating();
		long time = 0;
		commCreate = new CommCreateGradientThread(this, morphogen, time);
		exec.execute(commCreate);

	}

}
