package sim.app.socialagents;

import ec.util.*;
import ec.*;
import ec.vector.*;

import java.text.*;

public class CommSocialIndividual extends VectorIndividual {

	public final static String P_NUMPARAMS = "numparams";

	public static SingleControl[] controls;

	private boolean[] conditions;

	private double[] parameters;

	private int numParams;

	public double[] commParams;

	private static double[][] commParamsRanges;

	public CommSocialIndividual() {
		System.out.println("Creating new ind");
	}

	public CommSocialIndividual(CommSocialIndividual that) {
		this.commParams = new double[that.commParams.length];
		for (int i = 0; i < this.commParams.length; i++) {
			this.commParams[i] = that.commParams[i];
		}
	}


	public void setup(final EvolutionState state, final Parameter base) {
		super.setup(state, base);

		if (!(species instanceof SocialSpecies))
			state.output.fatal("SocialIndividual requires a SocialSpecies",
					base, defaultBase());

		numParams = state.parameters.getInt(base.push(P_NUMPARAMS), null, 0);
		commParams = new double[numParams];
		commParamsRanges = new double[numParams][2];
		// gradientDuration
		// propagationDelay
		// checkIfMessageDelay
		// gradientStrength
		// speedWhilePropagating

		commParamsRanges[0][0] = 100;
		commParamsRanges[0][1] = 1000;
		commParamsRanges[1][0] = 100;
		commParamsRanges[1][1] = 1000;
		commParamsRanges[2][0] = 1;
		commParamsRanges[2][1] = 1000;
		commParamsRanges[3][0] = 40;
		commParamsRanges[3][1] = 200;
		commParamsRanges[4][0] = 0;
		commParamsRanges[4][1] = 0.10;

	}

	public void reset(EvolutionState state, int thread) {
		for (int x = 0; x < commParams.length; x++) {
			commParams[x] = (state.random[thread].nextDouble()
					* (commParamsRanges[x][1] - commParamsRanges[x][0]))+commParamsRanges[x][0];
		}
		System.out.println(this);

	}

	public Parameter defaultBase() {
		return SocialDefaults.base().push("individual");
	}

	public boolean equals(Object o1) {
		CommSocialIndividual i1 = (CommSocialIndividual) o1;

		if (this.commParams.length != i1.commParams.length
				|| this.commParams != i1.commParams)
			return false;

		for (int i = 0; i < commParams.length; i++) {
			if (this.commParams[i] != i1.commParams[i])
				return false;
		}

		return true;
	}

	public int hashCode() {
		int hash = this.getClass().hashCode();
		hash = (hash << 1 | hash >>> 31) ^ commParams.hashCode();

		/*
		 * for(int x=0;x<parameters.length;x++) { long l =
		 * Double.doubleToLongBits(parameters[x]); hash = ( hash << 1 | hash
		 * >>> 31 ) ^ (int)((l >>> 16) & 0xFFFFFFF) ^ (int)(l & 0xFFFF); }
		 */

		return hash;
	}

	public String toString() {
		return getString();
	}

	public Object protoClone() throws CloneNotSupportedException {
		CommSocialIndividual myObj = (CommSocialIndividual) super.protoClone();
		myObj.commParams = (double[]) this.commParams.clone();
		return myObj;
	}

	public void defaultCrossover(EvolutionState state, int thread,
			VectorIndividual ind) {
		SocialSpecies s = (SocialSpecies) species;
		CommSocialIndividual i = (CommSocialIndividual) ind;
		double tmpD;
		int tmpI;
		int point;

		if (commParams.length != i.commParams.length) {
			state.output
					.fatal("Genome lengths are not the same for fixed-length vector crossover");
		}

		switch (s.crossoverType) {
		case VectorSpecies.C_ONE_POINT:
			point = state.random[thread]
					.nextInt((commParams.length / s.chunksize) + 1);
			for (int x = 0; x < point * s.chunksize; x++) {
				tmpD = i.commParams[x];
				i.commParams[x] = commParams[x];
				commParams[x] = tmpD;
			}

			break;
		case VectorSpecies.C_TWO_POINT:
			int point0 = state.random[thread]
					.nextInt((commParams.length / s.chunksize) + 1);
			point = state.random[thread]
					.nextInt((commParams.length / s.chunksize) + 1);
			if (point0 > point) {
				int p = point0;
				point0 = point;
				point = p;
			}
			for (int x = point0 * s.chunksize; x < point * s.chunksize; x++) {
				tmpD = i.commParams[x];
				i.commParams[x] = commParams[x];
				commParams[x] = tmpD;
			}

			break;
		case VectorSpecies.C_ANY_POINT:
			for (int x = 0; x < commParams.length / s.chunksize; x++) {
				if (state.random[thread].nextBoolean(s.crossoverProbability)) {
					for (int y = x * s.chunksize; y < (x + 1) * s.chunksize; y++) {
						tmpD = i.commParams[y];
						i.commParams[y] = commParams[y];
						commParams[y] = tmpD;
					}
					break;
				}
			}

		}
	}

	public void defaultMutate(EvolutionState state, int thread) {
		SocialSpecies s = (SocialSpecies) species;
		if (s.mutationProbability > 0.0) {
			for (int x = 0; x < commParams.length; x++) {
				if (state.random[thread].nextBoolean(s.mutationProbability)) {
					commParams[x] = commParamsRanges[x][0]
							+ state.random[thread].nextDouble()
							* (commParamsRanges[x][1] - commParamsRanges[x][0]);

				}
			}
		}
	}

	public void setParamValue(int index, double val) {

		this.commParams[index] = val;

	}

	/*
	 * returns a string representation of this SocialIndividual
	 */
	public String getString() {

		DecimalFormat df = new DecimalFormat("0.00000");
		String ret = "";

		for (int i = 0; i < this.commParams.length; i++) {
			ret += " " + df.format(this.commParams[i]);
		}

		return ret;

	}

}
