package sim.app.socialagents;

import java.util.*;
import java.io.*;
import sim.util.*;

public class ConfigParams{

	private Hashtable table;
	private String completeFile; // used when we want to write out the parameter file

	public ConfigParams(String fileName){

		BufferedReader br = null;
		String line;
		StringTokenizer tokenizer;
		String key, value;

		this.table = new Hashtable();
		this.completeFile = new String();

		/* If there is an error opening the parameter file, print out an error and
		 *   exit the constructor
		 **/
		try{
			br = new BufferedReader(new FileReader(fileName));
		}catch(IOException e){
			System.err.println("(ConfigParams.ConfigParams) ERROR: Could not open parameter file: " + fileName);
			return;
		}

		// read in the file
		try{

			/* Read in all of the parameters, and put the key/value pair in the hash table
			 * We will allow for comments in the parameter file
			 *   - Comments in the parameter file will be ignored
			 *   - Comments are lines that begin with a '#'
			 **/
			while((line = br.readLine()) != null){

				/* This is where the contents of the parameter file is saved
				 * We can write out this string at a later time
				 **/
				if(this.completeFile.equals("")){
					this.completeFile = line;
				}else{
					this.completeFile += "\n" + line;
				}

				// skip over comments
				if(line.length() == 0 || line.charAt(0) == '#') continue;

				tokenizer = new StringTokenizer(line, ":");

				key = tokenizer.nextToken().toUpperCase();
				value = tokenizer.nextToken().trim();

				this.table.put(key, value);

			}

		}catch(IOException e){
			System.err.println("(ConfigParams.ConfigParams) ERROR: An IOException was thrown while reading " + fileName);
		}

	}

	public void printParamsFile(String fileName){

		PrintWriter printWriter;

		try{

			printWriter = new PrintWriter(new BufferedWriter(new FileWriter(fileName)));
			printWriter.println(this.completeFile);
			printWriter.close();

		}catch(IOException e){
			System.err.println("(ConfigParams.printParamsFile) ERROR: An IOException was thrown while writing " + fileName);
		}

	}

	public int getParamInt(String key){

		if(!this.table.containsKey(key)) return -1;

		try{
			return (new Integer((String)this.table.get(key))).intValue();
		}catch(NumberFormatException e){

			System.err.println("(ConfigParams.getParamInt) ERROR: Value for " + key + " is not of type int");
			return -1;

		}

	}

	public long getParamLong(String key){

		if(!this.table.containsKey(key)) return -1;

		try{
			return (new Long((String)this.table.get(key))).longValue();
		}catch(NumberFormatException e){

			System.err.println("(ConfigParams.getParamLong) ERROR: Value for " + key + " is not of type long");
			return -1;

		}

	}

	public double getParamDouble(String key){

		if(!this.table.containsKey(key)) return 0.0;

		try{
			return (new Double((String)this.table.get(key))).doubleValue();
		}catch(NumberFormatException e){

			System.err.println("(ConfigParams.getParamDouble) ERROR: Value for " + key + " is not of type double");
			return 0.0;

		}

	}

	public boolean getParamBoolean(String key){

		if(!this.table.containsKey(key)) return false;

		try{
			return (new Boolean((String)this.table.get(key))).booleanValue();
		}catch(NumberFormatException e){

			System.err.println("(ConfigParams.getParamBoolean) ERROR: Value for " + key + " is not of type boolean");
			return false;

		}

	}

	public Double2D getParamDouble2D(String key){

		if(!this.table.containsKey(key)) return new Double2D();

		StringTokenizer t = new StringTokenizer((String)this.table.get(key));
		double x, y;

		try{

			x = Double.parseDouble(t.nextToken());
			y = Double.parseDouble(t.nextToken());

			return new Double2D(x, y);

		}catch(NumberFormatException e){

			System.err.println("(ConfigParams.getParamDouble2D) ERROR: Value for " + key + " is not of type Double2D");
			return new Double2D();

		}

	}

	public int[] getParamIntArray(String key){

		if(!this.table.containsKey(key)) return new int[]{};

		StringTokenizer t = new StringTokenizer((String)this.table.get(key));
		int[] array = new int[t.countTokens()];

		try{

			for(int i = 0; i < array.length; i++){
				array[i] = Integer.parseInt(t.nextToken());
			}
			return array;

		}catch(NumberFormatException e){

			System.err.println("(ConfigParams.getParamInt) ERROR: Value for " + key + " is not of type int array");
			return null;

		}

	}

}
