package sim.app.socialagents; 

import ec.*; 
import ec.util.*; 
import ec.simple.*;


public class Evaluator extends Problem implements SimpleProblemForm { 
	
	public static final String P_NAME = "evaluator";
	public static final String P_RUNLENGTH = "runlength";
	public static final String P_WIDTH = "width"; 
	public static final String P_HEIGHT = "height"; 
	public static final String P_NUMAGENTS = "num-agents"; 
	public static final String P_DEPLOYMENT_INTERVAL = "deployment-interval"; 
	public static final String P_SENSOR_RADIUS = "sensor-radius"; 
	public static final String P_SPEED = "agent-speed"; 
	public static final String P_RECORD = "record-intervals"; 
	
	int runLength, recordIntervals;
	int width, height; 
	int numAgents; 
	int sensorRadius; 
	double agentSpeed; 
	NonSocialAgentController controller; 

	
	public void setup( final EvolutionState state, final Parameter base )
	{
		super.setup( state, base );
		runLength = state.parameters.getInt(base.push(P_RUNLENGTH),null,0); 
		width = state.parameters.getInt(base.push(P_WIDTH),null,0); 
		height = state.parameters.getInt(base.push(P_HEIGHT),null,0); 
		numAgents = state.parameters.getInt(base.push(P_NUMAGENTS),null,0); 
		sensorRadius = state.parameters.getInt(base.push(P_SENSOR_RADIUS),null,0); 
		agentSpeed = state.parameters.getDouble(base.push(P_SPEED),null,0); 
		recordIntervals = state.parameters.getInt(base.push(P_RECORD),null,0); 
		
		controller = new NonSocialAgentController(new MersenneTwisterFast());
	}
	
	public Parameter defaultBase()
	{
		return SocialDefaults.base().push(P_NAME); 
	}
		
	public void evaluate(final EvolutionState state,
                         final Individual ind,
                         final int threadnum) { 
		
		if (ind.evaluated) return; 
		
		if (! (ind instanceof SocialIndividual)) 
			state.output.error("Whoa! Individual not instance of SocialIndividual");
		
		SocialIndividual ind2 = (SocialIndividual)ind;
			
		for (int x=0; x < ind2.controls.length; x++) 
			controller.addControl(ind2.controls[x]); 

		SocialSim sim = new SocialSim(System.currentTimeMillis(), controller, runLength, recordIntervals, "run.conf"); 
		
		sim.start(); 
		long steps=0;
		
		while ((steps = sim.schedule.getSteps()) <= runLength){
			if (! sim.schedule.step(sim)) break; 
			
			
		}
		
		((SimpleFitness)ind.fitness).setFitness(state,(float)sim.obtainFitnessForGA(), false); 
		ind2.evaluated = true; 
	}
	
	
	public void describe(final Individual ind, 
                         final EvolutionState state, 
                         final int threadnum, final int log,
                         final int verbosity)    { }
	
}