/* This class creates a sum of vectors parallel to the obstacles
 * Should give a right/left hand on wall type behavior
 **/
package sim.app.socialagents;

import sim.util.*;

public class FollowObstacleGoal extends Goal{

	public FollowObstacleGoal(){
	}

	public MutableDouble2D getVectorToGoal(CommAgent agent){

		MutableDouble2D agentPosition = new MutableDouble2D(agent.positionx, agent.positiony);
		MutableDouble2D agentVelocity = new MutableDouble2D(agent.velocityx, agent.velocityy);
		MutableDouble2D vector, closestPoint;
		Double2D obstaclePosition;
		double minDist;
		double tmpx, tmpy, magn;
		double comfortZoneSq = (agent.sensorDist * agent.sensorDist) * (agent.comfortZone * agent.comfortZone);

		closestPoint = new MutableDouble2D(Double.MAX_VALUE, Double.MAX_VALUE);
		minDist = Double.MAX_VALUE;

		// find the closest obstacle point
		for(int i = 0; i < agent.obstaclePositions.numObjs; i++){

			obstaclePosition = (Double2D)agent.obstaclePositions.objs[i];

			if(agentPosition.distanceSq(obstaclePosition) < minDist){
				closestPoint = new MutableDouble2D(obstaclePosition);
				minDist = agentPosition.distanceSq(obstaclePosition);
			}

		}

		/* if the agent doesn't see any obstacles, then return the zero vector
		 * otherwise, return a vector perpendicular to the agent and the obstacle
		 **/
		if(agent.obstaclePositions.numObjs == 0){
			return new MutableDouble2D(0.0, 0.0);
		}else{
			vector = new MutableDouble2D();

			vector.subtract(closestPoint, agentPosition);

			/* it should never occur that an agent is directly on top of an agent, but we'll
			 *   handle that case anyways
			 **/
			if(vector.lengthSq() == 0.0){
				return vector;
			}else{

				vector.normalize();

				/* Creates a vector perpendicular to the obstacle and the agent
				 * Because there are two perpendicular vectors, we choose the one
				 *   that changes the agent's velocity the least
				 **/
				if(agentVelocity.dot(new MutableDouble2D(vector.y, -vector.x).normalize()) < agentVelocity.dot(new MutableDouble2D(-vector.y, vector.x).normalize())){
					vector = new MutableDouble2D(-vector.y, vector.x);
				}else{
					vector = new MutableDouble2D(vector.y, -vector.x);
				}

				return vector;
			
			}
		}

	}

}
