package sim.app.socialagents;

import sim.engine.*;
import sim.portrayal.*;
import sim.util.*;
import java.awt.*;
import ec.util.*;

public class NonSocialAgent extends Agent{

	public NonSocialAgent(Double2D vel, Double2D pos, double sense, int agentSize, double speed, MersenneTwisterFast rng, AgentController control){

		super(vel, pos, agentSize, sense, new Color(0, 0, 0), speed);

		/* this is extremely ugly, but this is how the agent's control system
		 *   is initialized when the GA is being run
		 **/
		this.controller = control;

		/* if no GA is running, a default control system is loaded
		 **/
		if(this.controller == null) { 
			this.controller = new NonSocialAgentController(rng);

			boolean[] cond = new boolean[AgentController.NUM_CONDITIONS];
			double[] parameters = new double[AgentController.NUM_PARAMS];

			/*
			// evolved with obstacles
			parameters[AgentController.NG_RANDOMNESS] = 0.5;
			parameters[AgentController.NG_COMFORT]    = 0.90962;

			parameters[AgentController.G_AVOID_AGENT]     = .833;
			parameters[AgentController.G_AVOID_OBSTACLE]  = .890;
			parameters[AgentController.G_MOMENTUM]        = .995;
			parameters[AgentController.G_GOTO_AOI]        = .297;
			parameters[AgentController.G_FOLLOW_OBSTACLE] = .838;
			 */ 
			
			//parameters = new double[] {0.10522, 0.01279, 0.24175, 0.69506, 0.43949, 0.46338, 0.59891}; 

			// run 10 snake individual
			//parameters = new double[] {0.0, 0.48924, 0.95325, 0.95944, 0.69385, 0.12345, 0.353};

			// run 2 high randomness
			//parameters = new double[] {0.21438, 0.78781, 0.65168, 0.73110, 0.98174, 0.15165, 0.32781};

			parameters = new double[] {0.001515, 0.88162, 0.67661, 0.5, 0.80078, 0.13847, 0.22561, 0.0};

			//parameters = new double[] {0.1777, 0.29700, 0.28260, 0.09472, 0.79083, 0.00880, 0.59875};

			this.controller.addControl(cond, parameters);
		}
	}

	public void step(SimState state){

		// if the agent is dead, we can exit out of this function
		if(!super.isAlive()) return;

		super.step(state);

		SocialSim sim = (SocialSim) state;
		double radiusSq = super.radius*super.radius;
		MutableDouble2D newVel;

		/* if it is time to update the agent, then...
		 * update the agent's comfort zone
		 * obtain the Agents' new velocity from its controller
		 *
		 * the controller takes into account the agent's internal state
		 *   in order to figure out which set of control parameters it
		 *   should use
		 **/
		if(sim.schedule.getSteps() % this.updateVelocityInterval == 0){

			this.comfortZone = this.controller.getNonGoalParam(this, AgentController.NG_COMFORT);
			newVel = this.controller.getNewVelocity(this);

			if(newVel.lengthSq() != 0.0){
				newVel.normalize();
				newVel.multiplyIn(this.maxSpeed);
			}

			this.updateVelocity(newVel.x, newVel.y);

		}

		this.moveOneStep(state);

	}

	public void draw(Object object, Graphics2D graphics, DrawInfo2D info){
		super.draw(object, graphics, info);
	}

}
