package sim.app.socialagents;

import sim.util.*;
import ec.util.*;

public class NonSocialAgentController extends AgentController{

	private MersenneTwisterFast rng;

	public NonSocialAgentController(MersenneTwisterFast mtf){

		int maxNumStates = (int)Math.pow(2, AgentController.NUM_CONDITIONS);

		this.nonGoalParams = new double[maxNumStates][AgentController.NUM_NG_PARAMS];
		this.roles = new Role[maxNumStates];

		this.rng = mtf;

	}

	/* returns the new velocity of the agent based on the agent's internal states
	 *   and its environment
	 **/
	public MutableDouble2D getNewVelocity(CommAgent agent){

		MutableDouble2D velocity, randomVector;
		boolean[] currentCondition = new boolean[AgentController.NUM_CONDITIONS];
		int curConditionIndex;
		double randomx, randomy;

		// for each condition variable in AgentController, set the appropriate currentCondition
		//currentCondition[AgentController.BOOL_ON_AOI] = agent.getOnAOI();

		curConditionIndex = this.getConditionIndex(currentCondition);

		velocity = this.roles[curConditionIndex].getNewVelocity(agent);

		if(velocity.lengthSq() != 0.0){
			velocity.normalize();
		}

		/* compute the random vector
		 * the length of this random vector should be equal to the
		 *   value at this.nonGoalParams[curConditionIndex][AgentController.NG_RANDOMNESS]
		 **/
		randomx = this.rng.nextDouble() * ((this.rng.nextInt(2)==0)?1.0:-1.0);
		randomy = this.rng.nextDouble() * ((this.rng.nextInt(2)==0)?1.0:-1.0);
		randomVector = new MutableDouble2D(randomx, randomy);

		if(randomVector.lengthSq() != 0.0){
			randomVector.normalize();
			randomVector.multiplyIn(this.nonGoalParams[curConditionIndex][AgentController.NG_RANDOMNESS]);
		}

		velocity.addIn(randomVector);

		if(velocity.lengthSq() != 0.0){
			velocity.normalize();
		}

		return velocity;
	}

}
