/* This class contains an array of goals.  These goals specify the
 *   direction that an AV must go in order to best meet that goal.
 * Each goal also has a weight specified for it, which indicates how
 *   important that goal for this specific role.
 **/
package sim.app.socialagents;

import sim.util.*;

public class Role{

	protected Goal[] goals;
	protected double[] weights;
	private int goalCount;

	public Role(int numGoals){

		this.goals = new Goal[numGoals];
		this.weights = new double[numGoals];
		this.goalCount = 0;

	}

	/* adds a goal into the array if the goalCount does not exceed the length of the goals array
	 **/
	public void addGoal(Goal goal, double weight){

		if(this.goalCount == goals.length){
			System.err.println("(Role.addGoal) ERROR: Attempting to add too many goals to this role.");
		}else{
			this.goals[this.goalCount] = goal;
			this.weights[this.goalCount] = weight;
			this.goalCount++;
		}

	}

	public MutableDouble2D getNewVelocity(CommAgent agent){

		MutableDouble2D newVel = new MutableDouble2D(0.0, 0.0);
		for(int i = 0; i < this.goals.length; i++){
			newVel.addIn(this.goals[i].getVectorToGoal(agent).multiplyIn(this.weights[i]));
		}
		
		return newVel;
	}

}
