/* gets the average average fitness and average best fitness for an experiment (set of runs)
 **/

package sim.app.socialagents;

import java.io.*;
import java.util.*;
import java.text.DecimalFormat;

public class RunAggregator{

	public static void main(String[] args) throws Exception{

		if(args.length != 2){
			System.err.println("USAGE: java RunAggregator [run_num_start] [run_num_end]");
			return;
		}

		double[][] avgFitness;
		double[][] bestFitness;
		int runNumStart, runNumEnd;
		int numGenerations;
		ConfigParams params;
		BufferedReader br;
		StringTokenizer tokenizer;

		runNumStart = Integer.parseInt(args[0]);
		runNumEnd   = Integer.parseInt(args[1]);

		params = new ConfigParams("output/run" + runNumStart + "/params.txt");

		numGenerations = params.getParamInt("NUM_GENERATIONS");

		avgFitness = new double[numGenerations][runNumEnd-runNumStart+1];
		bestFitness = new double[numGenerations][runNumEnd-runNumStart+1];

		for(int curRun = runNumStart; curRun <= runNumEnd; curRun++){

			br = new BufferedReader(new FileReader("output/run" + curRun + "/fitness.txt"));

			//skip over blank line and header
			br.readLine();
			br.readLine();

			for(int curGen = 0; curGen < numGenerations; curGen++){
				tokenizer = new StringTokenizer(br.readLine());

				//skip over gen number;
				tokenizer.nextToken();

				avgFitness[curGen][curRun-runNumStart] = Double.parseDouble(tokenizer.nextToken());

				//skip over std dev
				tokenizer.nextToken();

				bestFitness[curGen][curRun-runNumStart] = Double.parseDouble(tokenizer.nextToken());

			}

		}

		for(int curGen = 0; curGen < numGenerations; curGen++){

			System.out.println(curGen + "   " + UtilityFunctions.getArrayAvg(avgFitness[curGen]) + "   " + UtilityFunctions.getArrayStdDev(avgFitness[curGen]) + "   " + UtilityFunctions.getArrayAvg(bestFitness[curGen]) + "   " + UtilityFunctions.getArrayStdDev(bestFitness[curGen]));

		}

	}

}
