/* takes a particular run and averages each of the evolved control parameters throughout each generation
 *
 * this program will end up printing out 2^NUM_CONDITIONS * (NUM_NG_PARAMS + NUM_G_PARAMS) * NUM_GENERATIONS values
 * [NUM_GENERATIONS * 2^NUM_CONDITIONS lines, each containing (NUM_NG_PARAMS + NUM_G_PARAMS) values]
 *
 * For right now, we assume NUM_CONDITIONS = 0
 **/

package sim.app.socialagents;

import java.io.*;
import java.util.*;
import java.text.DecimalFormat;

public class RunAnalyzer{

	public static void main(String[] args) throws Exception{

		if(args.length != 1){
			System.err.println("USAGE: java RunAnalyzer [run_num]");
			return;
		}

		int runNum = Integer.parseInt(args[0]);
		ConfigParams params = new ConfigParams("output/run" + runNum + "/params.txt");
		int popSize = params.getParamInt("POP_SIZE");
		int numGenerations = params.getParamInt("NUM_GENERATIONS");
		DecimalFormat df = new DecimalFormat("0.00000");
		StringTokenizer tokenizer;
		String str;
		int numCondCombinations = (int)Math.pow(2.0, AgentController.NUM_CONDITIONS);

		BufferedReader br = new BufferedReader(new FileReader("output/run" + runNum + "/fullRun.txt"));
		int gen = 0;
		int ind = 0;

		double[][][][] parameters = new double[numCondCombinations][numGenerations][AgentController.NUM_PARAMS][popSize];

		/* skip over date ran, rng_seed and the blank line
		 **/
		br.readLine();
		br.readLine();
		br.readLine();

		for(int curGen = 0; curGen < numGenerations; curGen++){
			for(int curIndiv = 0; curIndiv < popSize; curIndiv++){

				tokenizer = new StringTokenizer(br.readLine());

				// skip generation and gen#
				tokenizer.nextToken();
				tokenizer.nextToken();

				// skip individual and individual#
				tokenizer.nextToken();
				tokenizer.nextToken();

				// skip rng_seed and rng_seed#
				tokenizer.nextToken();
				tokenizer.nextToken();

				// skip fitness and fitness#
				tokenizer.nextToken();
				tokenizer.nextToken();

				for(int curCond = 0; curCond < numCondCombinations; curCond++){

					// skip CONDITION-->
					tokenizer.nextToken();

					for(int curParam = 0; curParam < AgentController.NUM_PARAMS; curParam++){
						parameters[curCond][curGen][curParam][curIndiv] = Double.parseDouble(tokenizer.nextToken());
					}

				}
			}
		}
		
		PrintWriter avgPR, stdDevPR;

		double[] parametersAvg = new double[AgentController.NUM_PARAMS];
		double[] parametersStdDev = new double[AgentController.NUM_PARAMS];

		avgPR = new PrintWriter(new BufferedWriter(new FileWriter("output/run" + runNum + "/averageEvolvedParams.txt")));
		stdDevPR = new PrintWriter(new BufferedWriter(new FileWriter("output/run" + runNum + "/stdDevEvolvedParams.txt")));

		for(int curCond = 0; curCond < numCondCombinations ; curCond++){

			avgPR.println(AgentController.getHeader());
			stdDevPR.println(AgentController.getHeader());

			for(int curGen = 0; curGen < numGenerations; curGen++){

				for(int curParam = 0; curParam < AgentController.NUM_PARAMS; curParam++){
					parametersAvg[curParam] = UtilityFunctions.getArrayAvg(parameters[curCond][curGen][curParam]);
					parametersStdDev[curParam] = UtilityFunctions.getArrayStdDev(parameters[curCond][curGen][curParam]);
				}

				avgPR.println(AgentController.getString(parametersAvg));
				stdDevPR.println(AgentController.getString(parametersStdDev));
			}
			avgPR.println();
			stdDevPR.println();

		}

		avgPR.close();
		stdDevPR.close();

	}

}
