package sim.app.socialagents;

import sim.engine.*;
import sim.portrayal.*;
import sim.util.*;

import java.awt.*;
import java.util.HashMap;
import java.util.Map;

import bioutil.Morphogen;
import bioutil.Message;
import ec.util.*;

import java.util.LinkedList;
import java.lang.Runnable;
import java.util.concurrent.*;
import java.util.Queue;
import java.util.ArrayList;

/**
 * Class representing a SocialAgent
 *
 * @author Adelein Rodriguez
 */

public class SocialAgent extends Agent{

	private MutableDouble2D currentGoal;
	private MutableDouble2D goalInMemory;
	public final SocialSim state;
	private MersenneTwisterFast rng;
	//	Gradient information
	public Map morphogenList;                   //	list of morphogens seen so far
	private boolean source;                     //	whether or not agent is a source of a gradient
	public static Map messages;
	private ReceiveThread receiveThread;
	ExecutorService exec;
	boolean hasGradientInfo; 					//	if agent is propagating a gradient right now
	String activeGradientName;
	Morphogen activeGradient;
	public boolean searchingMode;				//	If agent is searching for a source
	SocialAgent previousNeighbor;
	int numTimesNeighborSeen;
	boolean hasGoal;
	// Params that will be evolved
	public CommunicationParameters commParams;
	public int gradientStrength; 				//	this is the number of max hoops of the gradient
	double speedWhilePropagating; 				//	reduced speed once it receives a gradient
	public int gradientDuration;				//	number of times the source will propagate the gradient
	public int propagationDelay; 				//	time (SECONDS) between when agent propagates a gradients and 
												//	repeates the process
	public int checkIfMessageDelay; 			//	time (SECONDS) agent waits between checks in the main message
												//  table to see if
												//	messages have arrived for this agent
	public SocialAgent(Double2D vel, Double2D pos, double sense, int agentSize, double speed, MersenneTwisterFast rng, AgentController control, SocialSim state){

		super(vel, pos, agentSize, sense, new Color(0, 0, 0), speed);
		
		this.currentGoal = null;
		this.goalInMemory = null;
		this.rng=rng;

		// this is extremely ugly, but this is how the agent's control system
		// is initialized when the GA is being run
		this.controller = control;
		/* if no GA is running, a default control system is loaded
		 **/
		//if(this.controller == null) { 

			this.controller = new SocialAgentController(rng);

			boolean[] cond = new boolean[AgentController.NUM_CONDITIONS];
			double[] parameters = new double[AgentController.NUM_PARAMS];

			//parameters = new double[] {0.0, 0.92962, 0.81344, 0.16957, 0.65419, 0.03725, 0.34485, .05370};
			parameters = new double[] {0.001515, 0.88162, 0.67661, 0.5, 0.80078, 0.13847, 0.22561, 0.0};
			this.controller.addControl(cond, parameters);
	
		//}
		this.state=state;
			
		commParams = new CommunicationParameters(rng);
		//	init gradient parameters
		morphogenList=new HashMap();
		messages=new HashMap();
		source=false;	
		//gradientDuration=100;
		//propagationDelay=1000;
		//checkIfMessageDelay=1000;
		//gradientStrength=40;
		//speedWhilePropagating=0.05;
		
		exec = Executors.newCachedThreadPool();
		exec.execute(new ReceiveThread(this));
		hasGradientInfo=false;
		activeGradientName=null;
		hasGoal=false;
		previousNeighbor=null;
		numTimesNeighborSeen=0;
		activeGradient=null;
	}
	public MutableDouble2D getVectorToGoal2(Agent agent){

		MutableDouble2D agentPosition = new MutableDouble2D(agent.positionx, agent.positiony);
		MutableDouble2D vector, closestPoint;
		Double2D obstaclePosition;
		double minDist;
		double comfortZoneSq = (agent.sensorDist * agent.sensorDist) * (agent.comfortZone * agent.comfortZone);

		closestPoint = new MutableDouble2D(Double.MAX_VALUE, Double.MAX_VALUE);
		minDist = Double.MAX_VALUE;

		// find the closest obstacle point
		for(int i = 0; i < agent.obstaclePositions.numObjs; i++){

			obstaclePosition = (Double2D)agent.obstaclePositions.objs[i];

			if(agentPosition.distanceSq(obstaclePosition) < minDist){
				closestPoint = new MutableDouble2D(obstaclePosition);
				minDist = agentPosition.distanceSq(obstaclePosition);
			}

		}

		/* if the closest obstacle is getting within the agent's comfort zone, then ignore it
		 * otherwise, create a vector pointing away from that obstacle
		 **/
		if(minDist > comfortZoneSq){
			return new MutableDouble2D(0.0, 0.0);
		}else{
			vector = new MutableDouble2D();

			vector.subtract(agentPosition, closestPoint);

			if(vector.lengthSq() != 0.0){
				vector.normalize();
			}

			return vector;
		}

	}

	public MutableDouble2D getVectorToGoal(SocialAgent agent){
		
		double minDist = Double.MAX_VALUE;
		double comfortZoneSq = (agent.sensorDist * agent.sensorDist) * (agent.comfortZone * agent.comfortZone);
		MutableDouble2D scoutPosition = this.getPosition();
		MutableDouble2D neighborPosition = agent.getPosition();
		MutableDouble2D closestPoint;
		closestPoint = new MutableDouble2D(Double.MAX_VALUE, Double.MAX_VALUE);
		
		MutableDouble2D vector = new MutableDouble2D(0.0, 0.0);
		Double2D otherPosition;
		double tmpx, tmpy, magn;
		otherPosition = new Double2D(agent.positionx,agent.positiony);
		
		if(scoutPosition.distanceSq(neighborPosition) < minDist){
			closestPoint = new MutableDouble2D(neighborPosition);
			minDist = scoutPosition.distanceSq(neighborPosition);
		}

		tmpx = otherPosition.x - this.positionx;
		tmpy = otherPosition.y - this.positiony;
		magn = Math.sqrt((tmpx*tmpx)+(tmpy*tmpy));

		/* if the closest obstacle is getting within the agent's comfort zone, keep going
		 * towards it, otherwise, create a vector pointing away from that obstacle
		 **/
		if(minDist <= comfortZoneSq){

			//vector = new MutableDouble2D();

			//vector.subtract(scoutPosition, closestPoint);
			tmpx*=0.12;
			tmpy*=0.12;

		}
		//	 if the current best neighbor has a value worse than mine
		//	deviate the direction
		if(agent.activeGradient!=null && this.activeGradient!=null 
				&& agent.activeGradient.value < this.activeGradient.value){
			
			tmpx*=-1;
			tmpy*=-1;
		}
		
		
		if(magn != 0.0){
			tmpx/=magn;
			tmpy/=magn;

			vector.addIn(tmpx, tmpy);
		}
		

		if(vector.lengthSq() != 0.0){
			vector.normalize();
		}
		
		
		return vector;
	}
	
	public MutableDouble2D getRandomVelocity(){
		
		double randomx, randomy;
		MutableDouble2D randomVector = new MutableDouble2D();
		
		randomx = this.rng.nextDouble() * ((this.rng.nextInt(2)==0)?1.0:-1.0);
		randomy = this.rng.nextDouble() * ((this.rng.nextInt(2)==0)?1.0:-1.0);
		randomVector = new MutableDouble2D(randomx, randomy);

		/*if(randomVector.lengthSq() != 0.0){
			randomVector.normalize();
			randomVector.multiplyIn(this.nonGoalParams[curConditionIndex][AgentController.NG_RANDOMNESS]);
		}*/
		
		return randomVector;
	}

	public MutableDouble2D getNewGoal(){
		
		MutableDouble2D newVel=null;
		SocialAgent bestNeighbor;
		if(searchingMode && hasGradientInfo){
			bestNeighbor=this.getHighestGradientNeighbor(activeGradientName);
			if(bestNeighbor!=null){
				newVel=getVectorToGoal(bestNeighbor);
			}
		}
			
		return newVel;

	}
	public void step(SimState state){
		// if the agent is dead, we can exit out of this function
		if(!super.isAlive()) return;
		
		SocialSim sim = (SocialSim) state;
		MutableDouble2D newVel=null;
		super.step(state);
		this.obtainMemory(state);
		//this.sendMessage(state);
			
		/* if it is time to update the agent, then...
		 * update the agent's comfort zone
		 * obtain the Agents' new velocity from its controller
		 *
		 * the controller takes into account the agent's internal state
		 *   in order to figure out which set of control parameters it
		 *   should use
		 **/
		if(sim.schedule.getSteps() % this.updateVelocityInterval == 0){

			//SocialAgent bestNeighbor=this.getHighestGradientNeighbor(this+"");
			//	 if it doesnt know where to go tell it
			if((newVel=getNewGoal()) !=null && sim.schedule.getSteps() % 2 == 0){
				
				this.setHighlight(true);
				this.maxSpeed=4;
				hasGoal=true;

			}else{
				this.comfortZone = this.controller.getNonGoalParam(this, AgentController.NG_COMFORT);
				newVel = this.controller.getNewVelocity(this);
				
			}	
			
			if(newVel.lengthSq() != 0.0){
				newVel.normalize();
				newVel.multiplyIn(this.maxSpeed);
			}
			
			this.updateVelocity(newVel.x, newVel.y);

		}
//		If we have see an AOI then let others know
		if(this.getOnAOI() && !this.source){
			String gradientName=this+"";
			this.createGradient(new Morphogen(gradientName,(int)this.commParams.getGradientStrength()));
			
		}
		
		this.moveOneStep(state);
	}

	// sets goalInMemory if the agent is on top of an AOI
	// also removes the agents' current goal if the agent is on the current goal AOI
	private Bag nearbyAOIs;
	private void obtainMemory(SimState state){

		if(this.getOnAOI()){

			SocialSim sim = (SocialSim)state;
			nearbyAOIs = new Bag();

			sim.areasOfInterest.getObjectsWithinDistance(new Double2D(this.positionx,this.positiony),this.getSensorDist(),false,false,nearbyAOIs);

			for(int i = 0; i < nearbyAOIs.numObjs; i++){

				final AreaOfInterest aoi = (AreaOfInterest)(nearbyAOIs.objs[i]);

				// skip the region if it is not active
				if(!aoi.isActive()) continue;

				/* set the goal in the agent's memory to the AOI's center
				 **/
				if(aoi.pointInsideRegion(this.positionx, this.positiony)){

					// remove the current goal
					if(this.currentGoal != null && this.currentGoal.equals(aoi.getCenter())) this.currentGoal = null;

					this.goalInMemory = new MutableDouble2D(aoi.getCenter());
					break;

				}

			}

		}

	}
	// sends the goalInMemory to nearby agents
	//private Bag nearbyAgents;
	private void sendMessage(SimState state){

		if(this.goalInMemory != null){

			SocialSim sim = (SocialSim)state;
			Bag nearbyAgents = new Bag();

			sim.agents.getObjectsWithinDistance(new Double2D(this.positionx,this.positiony),this.getSensorDist(),false,false,nearbyAgents);
		
			// iterate through each nearby agent and check whether or not it is within sensor range of this
			for(int i = 0; i < nearbyAgents.numObjs; i++){

				if(!(nearbyAgents.objs[i] instanceof SocialAgent)) continue;

				final SocialAgent otherAgent = (SocialAgent)(nearbyAgents.objs[i]);
				final double dist = Math.sqrt((otherAgent.positionx-this.positionx)*(otherAgent.positionx-this.positionx)+(otherAgent.positiony-this.positiony)*(otherAgent.positiony-this.positiony));

				// send the message to the nearby agent
				if(dist < this.sensorDist + otherAgent.radius && otherAgent.isAlive() && otherAgent.isActivated() && dist != 0.0){
					otherAgent.receiveMessage(this.goalInMemory);
				}

			}
		}
	}

	public void receiveMessage(MutableDouble2D message){

		if(this.goalInMemory == null || !this.goalInMemory.equals(message)){
			this.currentGoal = new MutableDouble2D(message);
		}

	}

	public void draw(Object object, Graphics2D graphics, DrawInfo2D info){
		super.draw(object, graphics, info);
	}

	public MutableDouble2D getGoalPoint(){
		return this.currentGoal;
	}

	public MutableDouble2D getMemory(){
		return this.goalInMemory;
	}
	
	public Morphogen getActiveMorphogen(){
		
		return this.activeGradient;
	}

	//returns true if it will be able to propagate the signal
	public boolean receiveGradient(Morphogen morphogen){

		System.out.println(this +" Receiving signal "+morphogen.name+" "+morphogen.value);
		Morphogen existingMorphogen = (Morphogen)morphogenList.get(morphogen.name);
		//	if this morphogen has not been see before or if it has been seen
		//	but its value is greater than an existing morphogen saved value then save it 
		if(existingMorphogen==null || (existingMorphogen!=null && existingMorphogen.value < morphogen.value)){
			
			morphogenList.put(morphogen.name,morphogen);
			activeGradientName = morphogen.name;
			activeGradient=morphogen;
			System.out.println(this +" Saving "+morphogen.name+" "+morphogen.value);
			hasGradientInfo=true;
			if(!this.source && !this.searchingMode){
				this.setColor(new Color(5*morphogen.value,100,100));
				this.maxSpeed=this.commParams.getSpeedWhilePropagating();
			}
			if(morphogen.value==0)return false;
			else return true;
		}else{
			
			System.out.println(this+" Ignoring gradient");
			return false;
		}

	}

	public void createGradient(Morphogen morphogen){
		
		System.out.println(this +" Creating gradient");
		source=true;
		morphogenList.put(morphogen.name,morphogen);
		this.setHighlight(true);
		this.maxSpeed=this.commParams.getSpeedWhilePropagating();
		long time=0;
		exec.execute(new CreateGradientThread(this ,morphogen, time));
		
	}
	//	This function should consider neighbors withing an 
	//	r distance, not necessarily within sensor range. The idea
	//	is that we want to simulate an agent broadcasting a message
	//	to a medium, rather than to other agents specifically. 
	//	An agent within another agent's sensor range is a neighbor
	public ArrayList getNeighbors(){

		Bag nearbyAgents = new Bag();

		this.state.agents.getObjectsWithinDistance(new Double2D(this.positionx,this.positiony),this.getSensorDist(),false,false,nearbyAgents);
		ArrayList neighbors = new ArrayList <SocialAgent>();
		for(int i = 0; i < nearbyAgents.numObjs; i++){
			
			if(!(nearbyAgents.objs[i] instanceof SocialAgent) || nearbyAgents.objs[i] == this) continue;
			SocialAgent otherAgent = (SocialAgent)(nearbyAgents.objs[i]);
			neighbors.add(otherAgent);

		}
		return neighbors;
		
	}
	
	//----------These are the functions needed to do neighborhood querying----------//
	
	//	This function should consider neighbors within sensor range only. 
	//	An agent within another agent's sensor range is not a close neighbor
	public ArrayList getCloseNeighbors(){

		Bag nearbyAgents = new Bag();

		this.state.agents.getObjectsWithinDistance(new Double2D(this.positionx,this.positiony),this.getSensorDist(),false,false,nearbyAgents);
		ArrayList neighbors = new ArrayList <SocialAgent>();
		for(int i = 0; i < nearbyAgents.numObjs; i++){
			
			if(!(nearbyAgents.objs[i] instanceof SocialAgent) || nearbyAgents.objs[i] == this) continue;
			SocialAgent otherAgent = (SocialAgent)(nearbyAgents.objs[i]);
			final double dist = Math.sqrt((otherAgent.positionx-this.positionx)*(otherAgent.positionx-this.positionx)+(otherAgent.positiony-this.positiony)*(otherAgent.positiony-this.positiony));
			//	check to see if otherAgent is withing sensor distance
			//if(dist < this.sensorDist + otherAgent.radius && otherAgent.isAlive() && dist != 0.0){
				neighbors.add(otherAgent);
			//}

		}
		return neighbors;
		
	}
	//	This function should consider neighbors within sensor range only. 
	//	An agent within another agent's sensor range is not a close neighbor
	public double getNeighborsAverageGradient(String gradientName){

		double neighborsSum=0;
		ArrayList <SocialAgent>neighbors = new ArrayList<SocialAgent>();
		neighbors=this.getCloseNeighbors();
		for(SocialAgent a: neighbors){
			
			neighborsSum=+((Morphogen)a.morphogenList.get(gradientName)).value;
		}
		
		return neighborsSum/nearbyAgents.numObjs;

	}
	
	//	This function should consider neighbors within sensor range only. 
	//	An agent within another agent's sensor range is not a close neighbor
	public SocialAgent getHighestGradientNeighbor(String gradientName){

		double neighborsSum=0;
		ArrayList <SocialAgent>neighbors = new ArrayList<SocialAgent>();
		neighbors=this.getCloseNeighbors();
		int numNeighbors=0;
		SocialAgent best=null;
		if(neighbors.size()>0){
			best=(SocialAgent)neighbors.get(0);
			numNeighbors++;
			for(SocialAgent a: neighbors){
				Morphogen otherAgentMorphogen;
				//	if your neighbor has gradient info at all
				if((otherAgentMorphogen=(Morphogen)a.morphogenList.get(gradientName))!=null && 
						best.morphogenList.get(gradientName)!=null){
					//	if this gradient value is better
					numNeighbors++;
					if(((Morphogen)best.morphogenList.get(gradientName)).value >
						otherAgentMorphogen.value){
						best=a;
					}//end if
				}//end if
			}//	end for

			System.out.println("Picking "+best+ " from " + numNeighbors);
			return best;
		}
		else{ 
			System.out.println("Not Picking from " + numNeighbors);
			return null;}

	}
	
	

	//	This function should consider neighbors within sensor range only. 
	//	An agent within another agent's sensor range is not a close neighbor 
	public int getNumNeighbors(SimState state){
		return this.getCloseNeighbors().size();
	}
	
	
	public boolean followGradient(String gradient, double previousAverage){
		return this.getNeighborsAverageGradient(gradient)>previousAverage;
	}

}


class ReceiveThread implements Runnable{
	
	SocialAgent agent;
	Message message;

	public ReceiveThread(SocialAgent agent){
		
		this.agent=agent;

	}
	
	public void run(){
		
		while(true){
			while((message=(Message)agent.messages.get(agent))==null)
			{
				try{TimeUnit.MILLISECONDS.sleep(agent.checkIfMessageDelay);}
				catch(InterruptedException e){}

			}
		
			agent.messages.remove(agent);
			if(agent.receiveGradient(message.morphogen)){
				//	Propagate Message
					//create new time, dont care about time for now
					
					long releaseTime=0;
					for(int i=0; i<(int)agent.commParams.getGradientDuration(); i++){
						Morphogen decreasedMorphogen = new Morphogen();
						decreasedMorphogen.name=message.morphogen.name;
						decreasedMorphogen.value=message.morphogen.value;
						decreasedMorphogen.value--;
						Message messageToChildren = new Message(decreasedMorphogen,releaseTime);
						System.out.println(agent +" Sending  message" +messageToChildren+"to childen");
						ArrayList<SocialAgent> neighbors = agent.getNeighbors();
						for(SocialAgent a : neighbors){
							
							a.messages.put(a,messageToChildren);
						}
						getNewMessage(message);
						try{TimeUnit.MILLISECONDS.sleep((long)agent.commParams.getPropagationDelay());}
						catch(InterruptedException e){}
					}
					//remove gradient
					//agent.morphogenList.remove(message.morphogen.name);

			}//	end if
		}//end while
		
	}
	//	Check for a new message
	//	If this message is better than the previous one
	//	and it can be propagate exchange it with the old one
	//	else do nothing, agent will keep propagating old message
	boolean getNewMessage(Message oldMessage){
		
		Message newMessage;
		if((newMessage=(Message)agent.messages.get(agent))!=null){
			agent.messages.remove(agent);
			if(agent.receiveGradient(newMessage.morphogen)){
				oldMessage=newMessage;
				System.out.println("##########################Changing color##############################");
				return true;
			}
		}
		return false;
	}

}
//	This thread keeps sending a gradient
//	as long as the gradient duration of the source
//	agent is
class CreateGradientThread implements Runnable{
	
	SocialAgent agent;
	Morphogen morphogen;
	long time;

	public CreateGradientThread(SocialAgent agent, Morphogen morphogen, long time){
		
		this.agent=agent;
		this.morphogen=morphogen;
		this.time=time;

	}
	
	public void run(){
		
		ArrayList<SocialAgent> neighbors = agent.getNeighbors();
		
		Message messageToChildren;
		for(int i = 0; i< (int)agent.commParams.getGradientDuration(); i++){
			 messageToChildren= new Message(morphogen,time);
			 System.out.println("***************************************************************");
			for(SocialAgent a : neighbors){
				
				a.messages.put(a, messageToChildren);
			}
			
			try{ TimeUnit.MILLISECONDS.sleep((long)agent.commParams.getPropagationDelay());}
			catch(InterruptedException e){}

		}
		
		
	}

}
