package sim.app.socialagents;

import ec.util.*;
import ec.*; 
import ec.vector.*;

import java.text.*;

public class SocialIndividual extends VectorIndividual  {
	
	public static SingleControl[] controls; 
	private boolean[] conditions;
	private double[] parameters;
		
	public SocialIndividual(){ 
	}

	public SocialIndividual(SocialIndividual that){
			
		this.conditions = new boolean[that.conditions.length];
		this.parameters = new double[that.parameters.length];
		
		for(int i = 0; i < this.conditions.length; i++){
			this.conditions[i] = that.conditions[i];
		}
		
		for(int i = 0; i < this.parameters.length; i++){
			this.parameters[i] = that.parameters[i];
		}
	}
	
	public SocialIndividual(MersenneTwisterFast rng, int maxValue){
		
		this();

		for(int i = 0; i < this.conditions.length; i++){
			this.conditions[i] = rng.nextBoolean();
		}
		
		for(int i = 0; i < this.parameters.length; i++){
			this.parameters[i] = rng.nextDouble();
		}
	}
	
	public void setup(final EvolutionState state, final Parameter base)
		{
		super.setup(state,base); 
		
		if (!(species instanceof SocialSpecies))
			state.output.fatal("SocialIndividual requires a SocialSpecies", base, defaultBase()); 
				
		SocialSpecies s = (SocialSpecies)species; 		
		conditions = new boolean[s.numConditions]; 		
		parameters = new double[AgentController.NUM_PARAMS]; 
		controls = new SingleControl[(int)Math.pow(2,AgentController.NUM_CONDITIONS)]; 
		}
	
	public void reset(EvolutionState state, int thread)
		{
		SocialSpecies s = (SocialSpecies)species; 
			
		for (int x=0; x < conditions.length; x++) 
			conditions[x] = state.random[thread].nextBoolean(); 
		
		for (int x=0; x <  parameters.length; x++) 
			parameters[x] = s.minNonGoal + state.random[thread].nextDouble()*(s.maxNonGoal - s.minNonGoal); 
		
		for (int x=0; x < controls.length; x++ ) 
			controls[x] = new SingleControl(state.random[0], 1000); 
		
		}
	
	public Parameter defaultBase()
		{
		return SocialDefaults.base().push("individual"); 
		}
	
	public boolean equals(Object o1)
		{
		SocialIndividual i1 = (SocialIndividual) o1; 
		
		if (this.conditions.length != i1.conditions.length || 
			this.parameters != i1.parameters )
			return false; 
		
		for (int i=0; i < conditions.length; i++) { 
			if (this.conditions[i] != i1.conditions[i])
				return false; 
		}
		
		for (int i=0; i < parameters.length; i++) { 
			if (this.parameters[i] != i1.parameters[i])
				return false; 
		}
		
		return true;
		}
	
	public int hashCode()
		{
		int hash = this.getClass().hashCode(); 
		hash = (hash << 1 | hash >>> 31) ^ conditions.hashCode(); 
		
		for(int x=0;x<parameters.length;x++) {
            long l = Double.doubleToLongBits(parameters[x]);
            hash = ( hash << 1 | hash >>> 31 ) ^ (int)((l >>> 16) & 0xFFFFFFF) ^ (int)(l & 0xFFFF);
		}
		
		return hash; 		
		}
	
	public String toString ()
		{
		return getString(); 
		}
	
	public Object protoClone() throws CloneNotSupportedException
	{
		SocialIndividual myObj = (SocialIndividual)super.protoClone(); 
		myObj.conditions = (boolean[])conditions.clone(); 
		myObj.parameters = (double[])parameters.clone(); 
		myObj.controls = (SingleControl[])controls.clone(); 
		return myObj; 
	}
	
	
	public void defaultCrossover (EvolutionState state, int thread, VectorIndividual ind) 
		{
		SocialSpecies s = (SocialSpecies) species;
        SocialIndividual i = (SocialIndividual) ind;
        double tmpD;
		boolean tmpB; 
		int tmpI;
        int point;
		
		if (conditions.length != i.conditions.length || 
			parameters.length != i.parameters.length) 
			state.output.fatal("Genome lengths are not the same for fixed-length vector crossover");
        
		switch(s.crossoverType)
            {
            case VectorSpecies.C_ONE_POINT:
				point = state.random[thread].nextInt((conditions.length / s.chunksize)+1); 
				for (int x=0; x < point * s.chunksize; x++) { 
					tmpB = i.conditions[x]; 
					i.conditions[x] = conditions[x]; 
					conditions[x] = tmpB; 
				}
					
					point = state.random[thread].nextInt((parameters.length / s.chunksize)+1); 
				for (int x=0; x < point * s.chunksize; x++) { 
					tmpD = i.parameters[x]; 
					i.parameters[x] = parameters[x]; 
					parameters[x] = tmpD; 
				}
				break;
            case VectorSpecies.C_TWO_POINT: 
                int point0 = state.random[thread].nextInt((conditions.length / s.chunksize)+1);
                point = state.random[thread].nextInt((conditions.length / s.chunksize)+1);
                if (point0 > point) { int p = point0; point0 = point; point = p; }
					for(int x=point0*s.chunksize;x<point*s.chunksize;x++)
						{
						tmpB = i.conditions[x];
						i.conditions[x] = conditions[x];
						conditions[x] = tmpB;
						}
					
				point0 = state.random[thread].nextInt((parameters.length / s.chunksize)+1);
                point = state.random[thread].nextInt((parameters.length / s.chunksize)+1);
                if (point0 > point) { int p = point0; point0 = point; point = p; }
					for(int x=point0*s.chunksize;x<point*s.chunksize;x++)
						{
						tmpD = i.parameters[x];
						i.parameters[x] = parameters[x];
						parameters[x] = tmpD;
						}
				break;
            case VectorSpecies.C_ANY_POINT:
                for(int x=0;x<conditions.length/s.chunksize;x++) {
                    if (state.random[thread].nextBoolean(s.crossoverProbability)) { 
                        for(int y=x*s.chunksize;y<(x+1)*s.chunksize;y++)
                            {
                            tmpB = i.conditions[y];
                            i.conditions[y] = conditions[y];
                            conditions[y] = tmpB;
                            }
						break;
					}
				}
				
				for(int x=0;x<parameters.length/s.chunksize;x++) { 
                    if (state.random[thread].nextBoolean(s.crossoverProbability)) { 
                        for(int y=x*s.chunksize;y<(x+1)*s.chunksize;y++)
                            {
                            tmpD = i.parameters[y];
                            i.parameters[y] = parameters[y];
                            parameters[y] = tmpD;
                            }
						break;
					}
				}
            }
		}
	
	
	public void defaultMutate(EvolutionState state, int thread) 
	{
		SocialSpecies s = (SocialSpecies)species; 
		if (s.mutationProbability > 0.0) { 
			for (int x=0; x < conditions.length; x++) { 
				if (state.random[thread].nextBoolean(s.mutationProbability)) { 
					conditions[x] = !conditions[x]; 
				}
			}
			
			for (int x=0; x < parameters.length; x++) { 
				if (state.random[thread].nextBoolean(s.mutationProbability)) { 
					parameters[x] = s.minNonGoal + state.random[thread].nextDouble()*(s.maxNonGoal - s.minNonGoal); 

				
				}
			}
		}
	}
	
		
	public void setConditionValue(int index, boolean val){
		
		this.conditions[index] = val;
		
	}
	
	public void setNonGoalParam(int index, double val){
		
		this.parameters[index] = val;
		
	}
	
	/* returns a string representation of this SocialIndividual
		**/
	public String getString(){
		
		DecimalFormat df = new DecimalFormat("0.00000");
		String ret = "";
		
		for(int i = 0; i < this.conditions.length; i++){
			ret += (this.conditions[i]?"T":"F");
		}
		
		ret += "-->";
		
		for(int i = 0; i < this.parameters.length; i++){
			ret += " " + df.format(this.parameters[i]);
		}
		
		return ret;
		
	}
	
}
