 package sim.app.socialagents;

import sim.portrayal.continuous.*;
import sim.engine.*;
import sim.display.*;
import javax.swing.*;
import java.awt.*;

/**
 * @author Adelein Rodriguez
 */
public class SocialSimUI extends GUIState{

	public Display2D display;
	public JFrame displayFrame;

	ContinuousPortrayal2D agentPortrayal = new ContinuousPortrayal2D();
	ContinuousPortrayal2D obstaclePortrayal = new ContinuousPortrayal2D();
	ContinuousPortrayal2D areaOfInterestPortrayal = new ContinuousPortrayal2D();

	public static void main(String[] args){

		SocialSimUI simUI = new SocialSimUI();
		Console c = new Console(simUI);
		c.setVisible(true);
		
		

	}

	public SocialSimUI() { super(new CommSocialSim( System.currentTimeMillis()));
	
	
	}
	public SocialSimUI(SimState state) { super(state); }

	public String getName() { return "Social Agents"; }
	
	public String getInfo() { return "Gradient Simulation"; }

	public Object getSimulationInspectedObject() { return state; }

	public void start(){

		super.start();
		setupPortrayals();
		

	}

	public void load(SimState state){

		super.load(state);
		setupPortrayals();

	}

	public void setupPortrayals(){

		CommSocialSim sim = (CommSocialSim) state;
		
		// tell the portrayals what to portray and how to portray them
		agentPortrayal.setField(sim.agents);
		obstaclePortrayal.setField(sim.obstacles);
		areaOfInterestPortrayal.setField(sim.areasOfInterest);

		// reschedule the displayer
		display.reset();
		display.setBackdrop(Color.white);

		// redraw the display
		display.repaint();

	}

	public void init(Controller c){

		super.init(c);

		// make the displayer
		display = new Display2D(600,600,this,1);

		displayFrame = display.createFrame();
		displayFrame.setTitle("Social Agent Environment");
		c.registerFrame(displayFrame);   // register the frame so it appears in the "Display" list
		displayFrame.setVisible(true);
		display.attach(this.areaOfInterestPortrayal, "Areas of Interest");
		display.attach(this.obstaclePortrayal, "Obstacles");
		display.attach(this.agentPortrayal, "Agents");

		
	}

	public void quit(){

		super.quit();

		if (displayFrame!=null) displayFrame.dispose();
		displayFrame = null;
		display = null;

	}

}
